@@ -113,6 +113,7 @@ class Agent(Specifiable):
         # the Agent's policy Component.
         self.input_spaces = dict(
             states=self.state_space.with_batch_rank(),
+            time_percentage=float
         )
 
         # Construct the Preprocessor.
@@ -154,7 +155,7 @@ class Agent(Specifiable):
         # operations.
         self.loss_function = None
 
-        self.exploration = Exploration.from_spec(exploration_spec)
+        self.exploration = Exploration.from_spec(exploration_spec)  # TODO: Move this to DQN/DQFN. PG's don't use it.
         self.execution_spec = parse_execution_spec(execution_spec)
 
         # Python-side experience buffer for better performance (may be disabled).
@@ -283,7 +284,7 @@ class Agent(Specifiable):
                     executed before the increase takes place.
 
             Returns:
-                DataOp: no_op.
+                DataOp: no_op() or identity(other_step_op) in tf, None in pytorch.
             """
             if get_backend() == "tf":
                 add_op = tf.assign_add(self.graph_executor.global_training_timestep, 1)
@@ -340,7 +341,8 @@ class Agent(Specifiable):
             # Return identity.
             return states
 
-    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None):
+    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None,
+                   time_percentage=None):
         """
         Returns action(s) for the passed state(s). If `states` is a single state, returns a single action, otherwise,
         returns a batch of actions, where batch-size = number of states passed in.
@@ -497,7 +499,7 @@ class Agent(Specifiable):
         """
         raise NotImplementedError
 
-    def update(self, batch=None, **kwargs):
+    def update(self, batch=None, time_percentage=None, **kwargs):
         """
         Performs an update on the computation graph either via externally experience or
         by sampling from an internal memory.
@@ -506,6 +508,10 @@ class Agent(Specifiable):
             batch (Optional[dict]): Optional external data batch to use for update. If None, the
                 agent should be configured to sample internally.
 
+            time_percentage (Optional[float]): A percentage value (between 0.0 and 1.0) of the time already passed until
+                some max timesteps have been reached. This can be used by the algorithm to decay certain parameters
+                (e.g. learning rate) over time.
+
         Returns:
             Union(list, tuple, float): The loss value calculated in this update.
         """
