@@ -62,8 +62,6 @@ class DQFDAgent(Agent):
         memory_spec=None,
         demo_memory_spec=None,
         demo_sample_ratio=0.2,
-        store_last_memory_batch=False,
-        store_last_q_table=False
     ):
 
         """
@@ -98,16 +96,12 @@ class DQFDAgent(Agent):
             n_step (Optional[int]): n-step adjustment to discounting.
             memory_spec (Optional[dict,Memory]): The spec for the Memory to use.
             demo_memory_spec (Optional[dict,Memory]): The spec for the Demo-Memory to use.
-            store_last_memory_batch (bool): Whether to store the last pulled batch from the memory in
-                `self.last_memory_batch` for debugging purposes.
-                Default: False.
-            store_last_q_table (bool): Whether to store the Q(s,a) values for the last received batch
-                (memory or external) in `self.last_q_table` for debugging purposes.
-                Default: False.
         """
         # Fix action-adapter before passing it to the super constructor.
         # Use a DuelingPolicy (instead of a basic Policy) if option is set.
         if dueling_q is True:
+            if policy_spec is None:
+                policy_spec = {}
             policy_spec["type"] = "dueling-policy"
             # Give us some default state-value nodes.
             if "units_state_value_stream" not in policy_spec:
@@ -147,15 +141,9 @@ class DQFDAgent(Agent):
         self.default_margins = np.asarray([self.expert_margin] * self.batch_size)
 
         self.demo_batch_size = int(demo_sample_ratio * self.update_spec["batch_size"] / (1.0 - demo_sample_ratio))
-        self.demo_margins =  np.asarray([self.expert_margin] * self.demo_batch_size)
+        self.demo_margins = np.asarray([self.expert_margin] * self.demo_batch_size)
         self.shared_container_action_target = shared_container_action_target
 
-        # Debugging tools.
-        self.store_last_memory_batch = store_last_memory_batch
-        self.last_memory_batch = None
-        self.store_last_q_table = store_last_q_table
-        self.last_q_table = None
-
         # Extend input Space definitions to this Agent's specific API-methods.
         preprocessed_state_space = self.preprocessed_state_space.with_batch_rank()
         reward_space = FloatBox(add_batch_rank=True)
@@ -195,12 +183,12 @@ class DQFDAgent(Agent):
         # Number of steps since the last target-net synching from the main policy.
         self.steps_since_target_net_sync = 0
 
-        use_importance_weights = isinstance(self.memory, PrioritizedReplay)
+        self.use_importance_weights = isinstance(self.memory, PrioritizedReplay)
         self.loss_function = DQFDLossFunction(
             supervised_weight=supervised_weight,
             discount=self.discount, double_q=self.double_q, huber_loss=self.huber_loss,
             shared_container_action_target=shared_container_action_target,
-            importance_weights=use_importance_weights, n_step=n_step
+            importance_weights=self.use_importance_weights, n_step=n_step
         )
 
         # Add all our sub-components to the core.
@@ -269,41 +257,40 @@ class DQFDAgent(Agent):
 
         # Learn from online memory AND demo memory.
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(root, apply_demo_loss, expert_margins):
+        def update_from_memory(root, apply_demo_loss, expert_margins, time_percentage=None):
             # Sample from online memory.
             records, sample_indices, importance_weights = agent.memory.get_records(self.batch_size)
             preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.call(records)
 
             # Do not apply demo loss to online experience.
-            online_step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
+            online_step_op, loss, loss_per_item = root.update_from_external_batch(
                 preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss,
-                expert_margins
+                expert_margins, time_percentage
             )
 
             if isinstance(agent.memory, PrioritizedReplay):
                 update_pr_step_op = agent.memory.update_records(sample_indices, loss_per_item)
-
-                return online_step_op, loss, loss_per_item, records, q_values_s, update_pr_step_op
+                return online_step_op, loss, loss_per_item, update_pr_step_op
             else:
-                return online_step_op, loss, loss_per_item, records, q_values_s
+                return online_step_op, loss, loss_per_item
 
         # Learn from demo-data.
         @rlgraph_api(component=self.root_component)
-        def update_from_demos(root, demo_batch_size, apply_demo_loss, expert_margins):
+        def update_from_demos(root, demo_batch_size, apply_demo_loss, expert_margins, time_percentage=None):
             # Sample from demo memory.
             records, sample_indices, importance_weights = agent.demo_memory.get_records(demo_batch_size)
             preprocessed_s, actions, rewards, terminals, preprocessed_s_prime = agent.splitter.call(records)
             step_op, loss, loss_per_item, q_values_s = root.update_from_external_batch(
                 preprocessed_s, actions, rewards, terminals, preprocessed_s_prime, importance_weights, apply_demo_loss,
-                expert_margins
+                expert_margins, time_percentage
             )
-            return step_op, loss, loss_per_item, records, q_values_s
+            return step_op, loss, loss_per_item
 
         # Learn from an external batch - note the flag to apply demo loss.
         @rlgraph_api(component=self.root_component)
         def update_from_external_batch(
                 root, preprocessed_states, actions, rewards, terminals, preprocessed_next_states, importance_weights,
-                apply_demo_loss, expert_margins
+                apply_demo_loss, expert_margins, time_percentage=None
         ):
             # If we are a multi-GPU root:
             # Simply feeds everything into the multi-GPU sync optimizer's method and return.
@@ -313,7 +300,7 @@ class DQFDAgent(Agent):
                 grads_and_vars, loss, loss_per_item, q_values_s = \
                     root.sub_components["multi-gpu-synchronizer"].calculate_update_from_external_batch(
                         dict(policy=main_policy_vars), preprocessed_states, actions, rewards, terminals,
-                        preprocessed_next_states, importance_weights, apply_demo_loss, expert_margins
+                        preprocessed_next_states, importance_weights, apply_demo_loss, expert_margins, time_percentage
                     )
                 step_op = agent.optimizer.apply_gradients(grads_and_vars)
                 # Increase the global training step counter.
@@ -321,7 +308,7 @@ class DQFDAgent(Agent):
                 step_and_sync_op = root.sub_components["multi-gpu-synchronizer"].sync_variables_to_towers(
                     step_op, main_policy_vars
                 )
-                return step_and_sync_op, loss, loss_per_item, q_values_s
+                return step_and_sync_op, loss, loss_per_item
 
             # Get sub-components relative to the root (could be multi-GPU setup where root=some-tower).
             policy = root.get_sub_component_by_name(agent.policy.scope)
@@ -339,23 +326,24 @@ class DQFDAgent(Agent):
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins, q_values_sp,
-                importance_weights, apply_demo_loss
+                importance_weights, apply_demo_loss, time_percentage
             )
 
             # Args are passed in again because some device strategies may want to split them to different devices.
             policy_vars = policy.variables()
             if hasattr(root, "is_multi_gpu_tower") and root.is_multi_gpu_tower is True:
-                grads_and_vars = optimizer.calculate_gradients(policy_vars, loss)
-                return grads_and_vars, loss, loss_per_item, q_values_s
+                grads_and_vars = optimizer.calculate_gradients(policy_vars, loss, time_percentage)
+                return grads_and_vars, loss, loss_per_item
             else:
-                step_op, loss, loss_per_item = optimizer.step(policy_vars, loss, loss_per_item)
+                step_op = optimizer.step(policy_vars, loss, loss_per_item, time_percentage)
                 # Increase the global training step counter.
                 step_op = root._graph_fn_training_step(step_op)
-                return step_op, loss, loss_per_item, q_values_s
+                return step_op, loss, loss_per_item
 
         @rlgraph_api(component=self.root_component)
         def get_td_loss(root, preprocessed_states, actions, rewards,
-                        terminals, preprocessed_next_states, importance_weights, apply_demo_loss, expert_margins):
+                        terminals, preprocessed_next_states, importance_weights, apply_demo_loss, expert_margins,
+                        time_percentage=None):
 
             policy = root.get_sub_component_by_name(agent.policy.scope)
             target_policy = root.get_sub_component_by_name(agent.target_policy.scope)
@@ -371,7 +359,7 @@ class DQFDAgent(Agent):
 
             loss, loss_per_item = loss_function.loss(
                 q_values_s, actions, rewards, terminals, qt_values_sp, expert_margins,
-                q_values_sp, importance_weights, apply_demo_loss
+                q_values_sp, importance_weights, apply_demo_loss, time_percentage
             )
             return loss, loss_per_item
 
@@ -419,7 +407,8 @@ class DQFDAgent(Agent):
     def _observe_graph(self, preprocessed_states, actions, internals, rewards, next_states, terminals):
         self.graph_executor.execute(("insert_records", [preprocessed_states, actions, rewards, next_states, terminals]))
 
-    def update(self, batch=None, update_from_demos=False, expert_margins=None, apply_demo_loss_to_batch=False):
+    def update(self, batch=None, time_percentage=None, update_from_demos=False, expert_margins=None,
+               apply_demo_loss_to_batch=False):
         """
         Updates from external batch or replay memory.
         Args:
@@ -428,7 +417,7 @@ class DQFDAgent(Agent):
                 demo memory. Default false (only updating from main replay memory).
             expert_margins (SingleDataOp): The expert margin enforces a distance in Q-values between expert action and
                 all other actions.
-           apply_demo_loss_to_batch (bool): If true and an external batch is given, demo loss is applied to this
+            apply_demo_loss_to_batch (bool): If true and an external batch is given, demo loss is applied to this
                 batch. Can be combined with external per-sample expert margins. The purpose of this is to update
                 from external demonstration data where each sample has a different margin. If false and an
                 external batch is given, the agent updates this with the normal Double/Dueling-q loss. Default
@@ -436,6 +425,9 @@ class DQFDAgent(Agent):
         Returns:
             tuple: Loss and loss per item.
         """
+        # TODO: Move update_spec to Worker. Agent should not hold these execution details.
+        if time_percentage is None:
+            time_percentage = self.timesteps / self.update_spec.get("max_timesteps", 1e6)
 
         # Should we sync the target net?
         self.steps_since_target_net_sync += self.update_spec["update_interval"]
@@ -445,83 +437,57 @@ class DQFDAgent(Agent):
         else:
             sync_call = None
 
-        # [0]=no-op step; [1]=the loss; [2]=loss-per-item, [3]=memory-batch (if pulled); [4]=q-values
-        return_ops = [0, 1, 2]
-        q_table = None
-
         # Update from replay memory.  Potentially also update from demo memory with default margins.
         if batch is None:
-            # Add some additional return-ops to pull (left out normally for performance reasons).
-            if self.store_last_q_table is True:
-                return_ops += [3, 4]  # 3=batch, 4=q-values
-            elif self.store_last_memory_batch is True:
-                return_ops += [3]  # 3=batch
-
             # Combine: Update from memory (apply_demo_loss=False), update_from_demo (apply=True).
             # Otherwise only update from online memory.
             if update_from_demos:
                 # Use default margins whe sampling from memory.
-                ret = self.graph_executor.execute(("update_from_memory", [False, self.default_margins], return_ops),
-                                                  ("update_from_demos", [self.demo_batch_size, True,
-                                                                         self.demo_margins], return_ops),
-                                                  sync_call)
+                ret = self.graph_executor.execute(
+                    ("update_from_memory", [False, self.default_margins, time_percentage]),
+                    ("update_from_demos", [self.demo_batch_size, True, self.demo_margins, time_percentage]),
+                    sync_call
+                )
             else:
-                ret = self.graph_executor.execute(("update_from_memory",  [False, self.default_margins], return_ops),
-                                                  sync_call)
+                ret = self.graph_executor.execute(
+                    ("update_from_memory",  [False, self.default_margins, time_percentage]),
+                    sync_call
+                )
 
             # Remove unnecessary return dicts (e.g. sync-op).
             if isinstance(ret, dict):
                 ret = ret["update_from_memory"]
-
-            # Store the last Q-table?
-            if self.store_last_q_table is True:
-                q_table = dict(
-                    states=ret[3]["states"],
-                    q_values=ret[4]
-                )
         else:
             # Update from external batch, optionally applying demo loss. Also optionally
             # sample demos from separate demo memory.
-
-            # Add some additional return-ops to pull (left out normally for performance reasons).
-            if self.store_last_q_table is True:
-                return_ops += [3]  # 3=q-values
-
             if expert_margins is None:
                 # Default margins with correct len.
                 expert_margins = np.asarray([self.expert_margin] * len(batch["terminals"]))
 
             # Apply demo loss to external flag depending on batch.
             # Expert margins only have effect if True.
-            batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"],
-                           batch["next_states"], batch["importance_weights"], apply_demo_loss_to_batch, expert_margins]
+            batch_input = [
+                batch["states"], batch["actions"], batch["rewards"], batch["terminals"], batch["next_states"],
+                batch["importance_weights"], apply_demo_loss_to_batch, expert_margins, time_percentage
+            ]
 
             if update_from_demos:
-                ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops),
-                                                  ("update_from_demos", [self.demo_batch_size, True,
-                                                                         self.demo_margins], return_ops),
-                                                  sync_call)
+                ret = self.graph_executor.execute(
+                    ("update_from_external_batch", batch_input),
+                    ("update_from_demos", [self.demo_batch_size, True, self.demo_margins, time_percentage]),
+                    sync_call
+                )
             else:
                 # Only update from external batch (which may use demo loss depending on flag.
-                ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops), sync_call)
+                ret = self.graph_executor.execute(
+                    ("update_from_external_batch", batch_input),
+                    sync_call
+                )
 
             # Remove unnecessary return dicts (e.g. sync-op).
             if isinstance(ret, dict):
                 ret = ret["update_from_external_batch"]
 
-            # Store the last Q-table?
-            if self.store_last_q_table is True:
-                q_table = dict(
-                    states=batch["states"],
-                    q_values=ret[3]
-                )
-
-        # Store the latest pulled memory batch?
-        if self.store_last_memory_batch is True and batch is None:
-            self.last_memory_batch = ret[2]
-        if self.store_last_q_table is True:
-            self.last_q_table = q_table
-
         # [1]=the loss (0=update noop)
         # [2]=loss per item for external update, records for update from memory
         return ret[1], ret[2]
@@ -534,7 +500,7 @@ class DQFDAgent(Agent):
         if self.preprocessing_required and len(self.preprocessor.variable_registry) > 0:
             self.graph_executor.execute("reset_preprocessor")
 
-    def update_from_demos(self, num_updates=1, batch_size=None):
+    def update_from_demos(self, batch_size=None, time_percentage=None, num_updates=1):
         """
         Executes a number of updates by sampling from the expert memory.
 
@@ -543,13 +509,17 @@ class DQFDAgent(Agent):
             batch_size (Optional[int]): Sampling batch size to use. If None, uses the demo
                 batch size computed via the sampling ratio.
         """
+        # TODO: Move update_spec to Worker. Agent should not hold these execution details.
+        if time_percentage is None:
+            time_percentage = self.timesteps / self.update_spec.get("max_timesteps", 1e6)
+
         if batch_size is None:
             batch_size = self.demo_batch_size
             margins = self.demo_margins
         else:
             margins = np.asarray([self.expert_margin] * batch_size)
         for _ in range(num_updates):
-            self.graph_executor.execute(("update_from_demos", [batch_size, True, margins]))
+            self.graph_executor.execute(("update_from_demos", [batch_size, True, margins, time_percentage]))
 
     def observe_demos(self, preprocessed_states, actions, rewards, next_states, terminals):
         """
@@ -558,5 +528,6 @@ class DQFDAgent(Agent):
         self.graph_executor.execute(("insert_demos", [preprocessed_states, actions, rewards, next_states, terminals]))
 
     def __repr__(self):
-        return "DQFDAgent(doubleQ={} duelingQ={}, expert margin={})".format(self.double_q, self.dueling_q,
-                                                                            self.expert_margin)
+        return "DQFDAgent(doubleQ={} duelingQ={}, expert margin={})".format(
+            self.double_q, self.dueling_q, self.expert_margin
+        )
