@@ -21,7 +21,7 @@ import numpy as np
 
 from rlgraph import get_backend
 from rlgraph.agents import Agent
-from rlgraph.components import ContainerMerger, Memory, RingBuffer, PPOLossFunction
+from rlgraph.components import Memory, RingBuffer, PPOLossFunction
 from rlgraph.components.helpers import GeneralizedAdvantageEstimation
 from rlgraph.spaces import BoolBox, FloatBox
 from rlgraph.utils import util
@@ -77,33 +77,48 @@ class PPOAgent(Agent):
         Args:
             state_space (Union[dict,Space]): Spec dict for the state Space or a direct Space object.
             action_space (Union[dict,Space]): Spec dict for the action Space or a direct Space object.
+
             preprocessing_spec (Optional[list,PreprocessorStack]): The spec list for the different necessary states
                 preprocessing steps or a PreprocessorStack object itself.
+
             discount (float): The discount factor (gamma).
+
             network_spec (Optional[list,NeuralNetwork]): Spec list for a NeuralNetwork Component or the NeuralNetwork
                 object itself.
+
             internal_states_space (Optional[Union[dict,Space]]): Spec dict for the internal-states Space or a direct
                 Space object for the Space(s) of the internal (RNN) states.
+
             policy_spec (Optional[dict]): An optional dict for further kwargs passing into the Policy c'tor.
+
             value_function_spec (list, dict, ValueFunction): Neural network specification for baseline or instance
                 of ValueFunction.
+
             execution_spec (Optional[dict,Execution]): The spec-dict specifying execution settings.
             optimizer_spec (Optional[dict,Optimizer]): The spec-dict to create the Optimizer for this Agent.
+
             value_function_optimizer_spec (dict): Optimizer config for value function optimizer. If None, the optimizer
                 spec for the policy is used (same learning rate and optimizer type).
+
             observe_spec (Optional[dict]): Spec-dict to specify `Agent.observe()` settings.
             update_spec (Optional[dict]): Spec-dict to specify `Agent.update()` settings.
             summary_spec (Optional[dict]): Spec-dict to specify summary settings.
             saver_spec (Optional[dict]): Spec-dict to specify saver settings.
+
             auto_build (Optional[bool]): If True (default), immediately builds the graph using the agent's
                 graph builder. If false, users must separately call agent.build(). Useful for debugging or analyzing
                 components before building.
+
             name (str): Some name for this Agent object.
-            clip_ratio (float): Clipping parameter for likelihood ratio.
+            clip_ratio (float): Clipping parameter for importance sampling (IS) likelihood ratio.
             gae_lambda (float): Lambda for generalized advantage estimation.
-            clip_rewards (float): Reward clip value. If not 0, rewards will be clipped into this range.
+
+            clip_rewards (float): Reward clipping value. If not 0, rewards will be clipped within a +/- `clip_rewards`
+                range.
+
             value_function_clipping (Optional[float]): If not None, uses clipped value function objective. If None,
                 uses simple value function objective.
+
             standardize_advantages (bool): If true, standardize advantage values in update.
 
             sample_episodes (bool): If True, the update method interprets the batch_size as the number of
@@ -162,8 +177,6 @@ class PPOAgent(Agent):
             apply_postprocessing=bool
         ))
 
-        # The merger to merge inputs into one record Dict going into the memory.
-        self.merger = ContainerMerger("states", "actions", "rewards", "terminals")
         self.memory = Memory.from_spec(memory_spec)
         assert isinstance(self.memory, RingBuffer), "ERROR: PPO memory must be ring-buffer for episode-handling!"
 
@@ -187,7 +200,7 @@ class PPOAgent(Agent):
 
         # Add all our sub-components to the core.
         self.root_component.add_components(
-            self.preprocessor, self.merger, self.memory,
+            self.preprocessor, self.memory,
             self.policy, self.exploration,
             self.loss_function, self.optimizer, self.value_function, self.value_function_optimizer, self.vars_merger,
             self.vars_splitter, self.gae_function
@@ -233,7 +246,7 @@ class PPOAgent(Agent):
         # Insert into memory.
         @rlgraph_api(component=self.root_component)
         def insert_records(root, preprocessed_states, actions, rewards, terminals):
-            records = agent.merger.merge(preprocessed_states, actions, rewards, terminals)
+            records = dict(states=preprocessed_states, actions=actions, rewards=rewards, terminals=terminals)
             return agent.memory.insert_records(records)
 
         @rlgraph_api(component=self.root_component)
@@ -244,25 +257,25 @@ class PPOAgent(Agent):
 
         # Learn from memory.
         @rlgraph_api(component=self.root_component)
-        def update_from_memory(root, apply_postprocessing):
+        def update_from_memory(root, apply_postprocessing=True, time_percentage=None):
             if agent.sample_episodes:
                 records = agent.memory.get_episodes(self.update_spec["batch_size"])
             else:
                 records = agent.memory.get_records(self.update_spec["batch_size"])
 
             # Route to post process and update method.
-            # Use terminals as sequence indices.
-            sequence_indices = records["terminals"]
+            sequence_indices = records["terminals"]  # TODO: return correct seq-indices automatically from mem
             return root.update_from_external_batch(
                 records["states"], records["actions"], records["rewards"], records["terminals"],
-                sequence_indices, apply_postprocessing
+                sequence_indices, apply_postprocessing, time_percentage
             )
 
         # N.b. this is here because the iterative_optimization would need policy/losses as sub-components, but
         # multiple parents are not allowed currently.
         @rlgraph_api(component=self.root_component)
         def _graph_fn_update_from_external_batch(
-                root, preprocessed_states, actions, rewards, terminals, sequence_indices, apply_postprocessing
+                root, preprocessed_states, actions, rewards, terminals, sequence_indices, apply_postprocessing=True,
+                time_percentage=None
         ):
             """
             Calls iterative optimization by repeatedly sub-sampling.
@@ -279,28 +292,32 @@ class PPOAgent(Agent):
             value_function_optimizer = root.get_sub_component_by_name(agent.value_function_optimizer.scope)
             vars_merger = root.get_sub_component_by_name(agent.vars_merger.scope)
             gae_function = root.get_sub_component_by_name(agent.gae_function.scope)
+
             prev_log_probs = policy.get_log_likelihood(preprocessed_states, actions)["log_likelihood"]
+            prev_state_values = value_function.value_output(preprocessed_states)
 
             if get_backend() == "tf":
-                # Log probs before update.
-                prev_log_probs = tf.stop_gradient(prev_log_probs)
                 batch_size = tf.shape(preprocessed_states)[0]
-                prior_baseline_values = tf.stop_gradient(value_function.value_output(preprocessed_states))
 
-                # Advantages are based on prior baseline values.
+                # Log probs before update (stop-gradient as these are used in target term).
+                prev_log_probs = tf.stop_gradient(prev_log_probs)
+                # State values before update (stop-gradient as these are used in target term).
+                prev_state_values = tf.stop_gradient(prev_state_values)
+
+                # Advantages are based on previous state values.
                 advantages = tf.cond(
-                        pred=apply_postprocessing,
-                        true_fn=lambda: gae_function.calc_gae_values(
-                            prior_baseline_values, rewards, terminals, sequence_indices),
-                        false_fn=lambda: rewards
-                    )
-
+                    pred=apply_postprocessing,
+                    true_fn=lambda: gae_function.calc_gae_values(
+                        prev_state_values, rewards, terminals, sequence_indices
+                    ),
+                    false_fn=lambda: rewards
+                )
                 if self.standardize_advantages:
                     mean, std = tf.nn.moments(x=advantages, axes=[0])
                     advantages = (advantages - mean) / std
 
                 def opt_body(index_, loss_, loss_per_item_, vf_loss_, vf_loss_per_item_):
-                    start = tf.random_uniform(shape=(), minval=0, maxval=batch_size - 1, dtype=tf.int32)
+                    start = tf.random_uniform(shape=(), minval=0, maxval=batch_size, dtype=tf.int32)
                     indices = tf.range(start=start, limit=start + agent.sample_size) % batch_size
                     sample_states = tf.gather(params=preprocessed_states, indices=indices)
                     if isinstance(actions, ContainerDataOp):
@@ -311,15 +328,15 @@ class PPOAgent(Agent):
                     else:
                         sample_actions = tf.gather(params=actions, indices=indices)
 
-                    sample_prior_log_probs = tf.gather(params=prev_log_probs, indices=indices)
+                    sample_prev_log_probs = tf.gather(params=prev_log_probs, indices=indices)
                     sample_rewards = tf.gather(params=rewards, indices=indices)
                     sample_terminals = tf.gather(params=terminals, indices=indices)
                     sample_sequence_indices = tf.gather(params=sequence_indices, indices=indices)
                     sample_advantages = tf.gather(params=advantages, indices=indices)
-                    sample_advantages.set_shape((self.sample_size, ))
+                    sample_advantages.set_shape((agent.sample_size,))
 
-                    sample_baseline_values = value_function.value_output(sample_states)
-                    sample_prior_baseline_values = tf.gather(params=prior_baseline_values, indices=indices)
+                    sample_state_values = value_function.value_output(sample_states)
+                    sample_prev_state_values = tf.gather(params=prev_state_values, indices=indices)
 
                     # If we are a multi-GPU root:
                     # Simply feeds everything into the multi-GPU sync optimizer's method and return.
@@ -356,39 +373,30 @@ class PPOAgent(Agent):
                                 out["loss"] = root._graph_fn_training_step(out["loss"])
                             return index_ + 1, out["loss"], out["loss_per_item"], loss_vf, loss_per_item_vf
 
-                    policy_probs = policy.get_log_likelihood(sample_states, sample_actions)["log_likelihood"]
-                    baseline_values = value_function.value_output(tf.stop_gradient(sample_states))
-                    sample_rewards = tf.cond(
-                        pred=apply_postprocessing,
-                        true_fn=lambda: gae_function.calc_gae_values(
-                            baseline_values, sample_rewards, sample_terminals, sample_sequence_indices),
-                        false_fn=lambda: sample_rewards
-                    )
-                    sample_rewards.set_shape((agent.sample_size, ))
+                    sample_log_probs = policy.get_log_likelihood(sample_states, sample_actions)["log_likelihood"]
+
                     entropy = policy.get_entropy(sample_states)["entropy"]
 
                     loss, loss_per_item, vf_loss, vf_loss_per_item = \
                         loss_function.loss(
-                            policy_probs, sample_prior_log_probs,
-                            sample_baseline_values, sample_prior_baseline_values, sample_advantages,  entropy
+                            sample_log_probs, sample_prev_log_probs,
+                            sample_state_values, sample_prev_state_values, sample_advantages, entropy, time_percentage
                         )
 
                     if hasattr(root, "is_multi_gpu_tower") and root.is_multi_gpu_tower is True:
-                        policy_grads_and_vars = optimizer.calculate_gradients(policy.variables(), loss)
+                        policy_grads_and_vars = optimizer.calculate_gradients(policy.variables(), loss, time_percentage)
                         vf_grads_and_vars = value_function_optimizer.calculate_gradients(
-                            value_function.variables(), vf_loss
+                            value_function.variables(), vf_loss, time_percentage
                         )
                         grads_and_vars_by_component = vars_merger.merge(policy_grads_and_vars, vf_grads_and_vars)
                         return grads_and_vars_by_component, loss, loss_per_item, vf_loss, vf_loss_per_item
                     else:
-                        step_op, loss, loss_per_item = optimizer.step(
-                            policy.variables(), loss, loss_per_item
-                        )
+                        step_op = optimizer.step(policy.variables(), loss, loss_per_item, time_percentage)
                         loss.set_shape(())
                         loss_per_item.set_shape((agent.sample_size,))
 
-                        vf_step_op, vf_loss, vf_loss_per_item = value_function_optimizer.step(
-                            value_function.variables(), vf_loss, vf_loss_per_item
+                        vf_step_op = value_function_optimizer.step(
+                            value_function.variables(), vf_loss, vf_loss_per_item, time_percentage
                         )
                         vf_loss.set_shape(())
                         vf_loss_per_item.set_shape((agent.sample_size,))
@@ -421,17 +429,17 @@ class PPOAgent(Agent):
                     return loss, loss_per_item, vf_loss, vf_loss_per_item
 
             elif get_backend() == "pytorch":
+                batch_size = preprocessed_states.shape[0]
+                sample_size = min(batch_size, agent.sample_size)
+
                 if isinstance(prev_log_probs, dict):
                     for name in actions.keys():
                         prev_log_probs[name] = prev_log_probs[name].detach()
                 else:
                     prev_log_probs = prev_log_probs.detach()
-                batch_size = preprocessed_states.shape[0]
-                sample_size = min(batch_size, agent.sample_size)
-                prior_baseline_values = value_function.value_output(preprocessed_states).detach()
+                prev_state_values = value_function.value_output(preprocessed_states).detach()
                 if apply_postprocessing:
-                    advantages = gae_function.calc_gae_values(
-                        prior_baseline_values, rewards, terminals, sequence_indices)
+                    advantages = gae_function.calc_gae_values(prev_state_values, rewards, terminals, sequence_indices)
                 else:
                     advantages = rewards
                 if self.standardize_advantages:
@@ -444,33 +452,33 @@ class PPOAgent(Agent):
 
                     if isinstance(actions, dict):
                         sample_actions = DataOpDict()
-                        sample_prior_log_probs = DataOpDict()
+                        sample_prev_log_probs = DataOpDict()
                         for name, action in define_by_run_flatten(actions, scope_separator_at_start=False).items():
                             sample_actions[name] = torch.index_select(action, 0, indices)
-                            sample_prior_log_probs[name] = torch.index_select(prev_log_probs[name], 0, indices)
+                            sample_prev_log_probs[name] = torch.index_select(prev_log_probs[name], 0, indices)
                     else:
                         sample_actions = torch.index_select(actions, 0, indices)
-                        sample_prior_log_probs = torch.index_select(prev_log_probs, 0, indices)
+                        sample_prev_log_probs = torch.index_select(prev_log_probs, 0, indices)
 
                     sample_advantages = torch.index_select(advantages, 0, indices)
-                    sample_prior_baseline_values = torch.index_select(prior_baseline_values, 0, indices)
+                    sample_prev_state_values = torch.index_select(prev_state_values, 0, indices)
 
-                    policy_probs = policy.get_log_likelihood(sample_states, sample_actions)["log_likelihood"]
-                    sample_baseline_values = value_function.value_output(sample_states)
+                    sample_log_probs = policy.get_log_likelihood(sample_states, sample_actions)["log_likelihood"]
+                    sample_state_values = value_function.value_output(sample_states)
 
                     entropy = policy.get_entropy(sample_states)["entropy"]
                     loss, loss_per_item, vf_loss, vf_loss_per_item = loss_function.loss(
-                        policy_probs, sample_prior_log_probs,
-                        sample_baseline_values,  sample_prior_baseline_values, sample_advantages, entropy
+                        sample_log_probs, sample_prev_log_probs,
+                        sample_state_values, sample_prev_state_values, sample_advantages, entropy, time_percentage
                     )
 
                     # Do not need step op.
-                    _, loss, loss_per_item = optimizer.step(policy.variables(), loss, loss_per_item)
-                    _, vf_loss, vf_loss_per_item = \
-                        value_function_optimizer.step(value_function.variables(), vf_loss, vf_loss_per_item)
+                    optimizer.step(policy.variables(), loss, loss_per_item, time_percentage)
+                    value_function_optimizer.step(value_function.variables(), vf_loss, vf_loss_per_item, time_percentage)
                 return loss, loss_per_item, vf_loss, vf_loss_per_item
 
-    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None):
+    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None,
+                   time_percentage=None):
         """
         Args:
             extra_returns (Optional[Set[str],str]): Optional string or set of strings for additional return
@@ -516,10 +524,9 @@ class PPOAgent(Agent):
     def _observe_graph(self, preprocessed_states, actions, internals, rewards, next_states, terminals):
         self.graph_executor.execute(("insert_records", [preprocessed_states, actions, rewards, terminals]))
 
-    def update(self, batch=None, sequence_indices=None, apply_postprocessing=True):
+    def update(self, batch=None, time_percentage=None, sequence_indices=None, apply_postprocessing=True):
         """
         Args:
-            batch (dict): Update batch.
             sequence_indices (Optional[np.ndarray, list]): Sequence indices are used in multi-env batches where
                 partial episode fragments may be concatenated within the trajectory. For a single env, these are equal
                 to terminals. If None are given, terminals will be used as sequence indices. A sequence index is True
@@ -535,11 +542,14 @@ class PPOAgent(Agent):
                 been applied. The purpose of internal versus external post-processing is to be able to off-load
                 post-processing in large scale distributed scenarios.
         """
+        # TODO: Move update_spec to Worker. Agent should not hold these execution details.
+        if time_percentage is None:
+            time_percentage = self.timesteps / self.update_spec.get("max_timesteps", 1e6)
 
         # [0] = the loss; [1] = loss-per-item, [2] = vf-loss, [3] = vf-loss- per item
         return_ops = [0, 1, 2, 3]
         if batch is None:
-            ret = self.graph_executor.execute(("update_from_memory", [True], return_ops))
+            ret = self.graph_executor.execute(("update_from_memory", [True, time_percentage], return_ops))
 
             # Remove unnecessary return dicts (e.g. sync-op).
             if isinstance(ret, dict):
@@ -551,20 +561,16 @@ class PPOAgent(Agent):
 
             pps_dtype = self.preprocessed_state_space.dtype
             batch["states"] = np.asarray(batch["states"], dtype=util.convert_dtype(dtype=pps_dtype, to='np'))
-            batch_input = [batch["states"], batch["actions"], batch["rewards"], batch["terminals"],
-                           sequence_indices, apply_postprocessing]
 
-            # Execute post-processing or already post-processed by workers?
-            if apply_postprocessing:
-                ret = self.graph_executor.execute(("post_process_and_update", batch_input, return_ops))
-                # Remove unnecessary return dicts (e.g. sync-op).
-                if isinstance(ret, dict):
-                    ret = ret["post_process_and_update"]
-            else:
-                ret = self.graph_executor.execute(("update_from_external_batch", batch_input, return_ops))
-                # Remove unnecessary return dicts (e.g. sync-op).
-                if isinstance(ret, dict):
-                    ret = ret["update_from_external_batch"]
+            ret = self.graph_executor.execute(
+                ("update_from_external_batch", [
+                    batch["states"], batch["actions"], batch["rewards"], batch["terminals"], sequence_indices,
+                    apply_postprocessing, time_percentage
+                ], return_ops)
+            )
+            # Remove unnecessary return dicts (e.g. sync-op).
+            if isinstance(ret, dict):
+                ret = ret["update_from_external_batch"]
 
         # [0] loss, [1] loss per item
         return ret[0], ret[1]
