@@ -143,11 +143,11 @@ class SACAgentComponent(Component):
         return self._memory.insert_records(records)
 
     @rlgraph_api
-    def update_from_memory(self, batch_size=64):
+    def update_from_memory(self, batch_size=64, time_percentage=None):
         records, sample_indices, importance_weights = self._memory.get_records(batch_size)
         result = self.update_from_external_batch(
             records["states"], records["actions"], records["rewards"], records["terminals"],
-            records["next_states"], importance_weights
+            records["next_states"], importance_weights, time_percentage
         )
 
         if isinstance(self._memory, PrioritizedReplay):
@@ -158,7 +158,7 @@ class SACAgentComponent(Component):
 
     @rlgraph_api
     def update_from_external_batch(
-        self, preprocessed_states, env_actions, rewards, terminals, next_states, importance_weights
+        self, preprocessed_states, env_actions, rewards, terminals, next_states, importance_weights, time_percentage
     ):
         actions = self._graph_fn_one_hot(env_actions)
         actor_loss, actor_loss_per_item, critic_loss, critic_loss_per_item, alpha_loss, alpha_loss_per_item = \
@@ -167,14 +167,11 @@ class SACAgentComponent(Component):
         policy_vars = self._policy.variables()
         q_vars = [q_func.variables() for q_func in self._q_functions]
         merged_q_vars = self._q_vars_merger.merge(*q_vars)
-        critic_step_op, critic_loss, critic_loss_per_item = \
-            self.vf_optimizer.step(merged_q_vars, critic_loss, critic_loss_per_item)
-
-        actor_step_op, actor_loss, actor_loss_per_item = \
-            self._optimizer.step(policy_vars, actor_loss, actor_loss_per_item)
+        critic_step_op = self.vf_optimizer.step(merged_q_vars, critic_loss, critic_loss_per_item, time_percentage)
+        actor_step_op = self._optimizer.step(policy_vars, actor_loss, actor_loss_per_item, time_percentage)
 
         if self.target_entropy is not None:
-            alpha_step_op = self._graph_fn_update_alpha(alpha_loss, alpha_loss_per_item)
+            alpha_step_op = self._graph_fn_update_alpha(alpha_loss, alpha_loss_per_item, time_percentage)
         else:
             alpha_step_op = self._graph_fn_no_op()
         # TODO: optimizer for alpha
@@ -204,9 +201,10 @@ class SACAgentComponent(Component):
         return env_actions
 
     @graph_fn(requires_variable_completeness=True)
-    def _graph_fn_update_alpha(self, alpha_loss, alpha_loss_per_item):
-        alpha_step_op, _, _ = self.alpha_optimizer.step(
-            DataOpTuple([self.log_alpha]), alpha_loss, alpha_loss_per_item)
+    def _graph_fn_update_alpha(self, alpha_loss, alpha_loss_per_item, time_percentage=None):
+        alpha_step_op = self.alpha_optimizer.step(
+            DataOpTuple([self.log_alpha]), alpha_loss, alpha_loss_per_item, time_percentage
+        )
         return alpha_step_op
 
     @rlgraph_api  # `returns` are determined in ctor
@@ -547,7 +545,8 @@ class SACAgent(Agent):
     def get_weights(self):
         return dict(policy_weights=self.graph_executor.execute(self.root_component.get_policy_weights))
 
-    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None):
+    def get_action(self, states, internals=None, use_exploration=True, apply_preprocessing=True, extra_returns=None,
+                   time_percentage=None):
         # TODO: common pattern - move to Agent
         """
         Args:
@@ -605,7 +604,7 @@ class SACAgent(Agent):
     def _observe_graph(self, preprocessed_states, actions, internals, rewards, next_states, terminals):
         self.graph_executor.execute((self.root_component.insert_records, [preprocessed_states, actions, rewards, next_states, terminals]))
 
-    def update(self, batch=None):
+    def update(self, batch=None, time_percentage=None, **kwargs):
         if batch is None:
             size = self.graph_executor.execute(self.root_component.get_memory_size)
             # TODO: is this necessary?
