@@ -18,9 +18,9 @@ from __future__ import division
 from __future__ import print_function
 
 from rlgraph import get_backend
-from rlgraph.components.helpers import GeneralizedAdvantageEstimation
+from rlgraph.components.common.time_dependent_parameters import TimeDependentParameter
 from rlgraph.components.loss_functions import LossFunction
-from rlgraph.spaces import IntBox, ContainerSpace
+from rlgraph.spaces import IntBox
 from rlgraph.spaces.space_utils import sanity_check_space
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
 
@@ -44,22 +44,29 @@ class ActorCriticLossFunction(LossFunction):
         L[E] = - SUM[all actions a] pi(a|s) * log pi(a|s)
 
     """
-    def __init__(self, weight_pg=None, weight_baseline=None, weight_entropy=None, **kwargs):
+    def __init__(self, weight_pg=None, weight_vf=None, weight_entropy=None, **kwargs):
         """
         Args:
             discount (float): The discount factor (gamma) to use.
             gae_lambda (float): Optional GAE discount factor.
             reward_clipping (Optional[str]): One of None, "clamp_one" or "soft_asymmetric". Default: "clamp_one".
             weight_pg (float): The coefficient used for the policy gradient loss term (L[PG]).
-            weight_baseline (float): The coefficient used for the Value-function baseline term (L[V]).
+            weight_vf (float): The coefficient used for the value function term (L[V]).
             weight_entropy (float): The coefficient used for the entropy regularization term (L[E]).
                 In the paper, values between 0.01 and 0.00005 are used via log-uniform search.
         """
         super(ActorCriticLossFunction, self).__init__(scope=kwargs.pop("scope", "actor-critic-loss-func"), **kwargs)
 
-        self.weight_pg = weight_pg if weight_pg is not None else 1.0
-        self.weight_baseline = weight_baseline if weight_baseline is not None else 0.5
-        self.weight_entropy = weight_entropy if weight_entropy is not None else 0.00025
+        self.weight_pg = TimeDependentParameter.from_spec(weight_pg if weight_pg is not None else 1.0,
+                                                          scope="weight-pg")
+        self.weight_vf = TimeDependentParameter.from_spec(
+            weight_vf if weight_vf is not None else 0.5, scope="weight-vf"
+        )
+        self.weight_entropy = TimeDependentParameter.from_spec(
+            weight_entropy if weight_entropy is not None else 0.00025, scope="weight-entropy"
+        )
+
+        self.add_components(self.weight_pg, self.weight_vf, self.weight_entropy)
 
     def check_input_spaces(self, input_spaces, action_space=None):
         assert action_space is not None
@@ -70,7 +77,7 @@ class ActorCriticLossFunction(LossFunction):
         )
 
     @rlgraph_api
-    def loss(self, log_probs, values, rewards, entropy):
+    def loss(self, log_probs, values, rewards, entropy, time_percentage=None):
         """
         API-method that calculates the total loss (average over per-batch-item loss) from the original input to
         per-item-loss.
@@ -81,7 +88,7 @@ class ActorCriticLossFunction(LossFunction):
             SingleDataOp: The tensor specifying the final loss (over the entire batch).
         """
         loss_per_item, vf_loss_per_item = self.loss_per_item(
-            log_probs, values, rewards, entropy
+            log_probs, values, rewards, entropy, time_percentage
         )
         total_loss = self.loss_average(loss_per_item)
         vf_total_loss = self.loss_average(vf_loss_per_item)
@@ -89,19 +96,19 @@ class ActorCriticLossFunction(LossFunction):
         return total_loss, loss_per_item, vf_total_loss, vf_loss_per_item
 
     @rlgraph_api
-    def loss_per_item(self, log_probs, baseline_values, advantages, entropy):
+    def loss_per_item(self, log_probs, state_values, advantages, entropy, time_percentage=None):
         # Get losses for each action.
         # Baseline loss for V(s) does not depend on actions, only on state.
-        baseline_loss_per_item = self._graph_fn_baseline_loss_per_item(baseline_values, advantages)
-        loss_per_item = self._graph_fn_loss_per_item(log_probs, advantages, entropy)
+        vf_loss_per_item = self._graph_fn_state_value_function_loss_per_item(state_values, advantages, time_percentage)
+        loss_per_item = self._graph_fn_loss_per_item(log_probs, advantages, entropy, time_percentage)
 
         # Average across actions.
         loss_per_item = self._graph_fn_average_over_container_keys(loss_per_item)
 
-        return loss_per_item, baseline_loss_per_item
+        return loss_per_item, vf_loss_per_item
 
     @graph_fn(flatten_ops=True, split_ops=True)
-    def _graph_fn_loss_per_item(self, log_probs, advantages, entropy):
+    def _graph_fn_loss_per_item(self, log_probs, advantages, entropy, time_percentage):
         """
         Calculates the loss per batch item (summed over all timesteps) using the formula described above in
         the docstring to this class.
@@ -110,6 +117,7 @@ class ActorCriticLossFunction(LossFunction):
             log_probs (DataOp): Log-likelihood of actions.
             advantages (DataOp): The received rewards.
             entropy (DataOp): Policy entropy
+
         Returns:
             SingleDataOp: The loss values per item in the batch, but summed over all timesteps.
         """
@@ -117,34 +125,24 @@ class ActorCriticLossFunction(LossFunction):
             # # Let the gae-helper function calculate the pg-advantages.
             advantages = tf.stop_gradient(advantages)
 
-            # The policy gradient loss.
-            loss = advantages * -log_probs
-            if self.weight_pg != 1.0:
-                loss = self.weight_pg * loss
-
-            # The entropy regularizer term.
-            loss += self.weight_entropy * entropy
-
-            return loss
         elif get_backend() == "pytorch":
             advantages = advantages.detach()
 
-            # The policy gradient loss.
-            loss = advantages * log_probs
-            if self.weight_pg != 1.0:
-                loss = self.weight_pg * loss
+        # The policy gradient loss.
+        loss = advantages * -log_probs
+        loss = self.weight_pg.get(time_percentage) * loss
 
-            # The entropy regularizer term.
-            loss += self.weight_entropy * entropy
-            return loss
+        # Subtract the entropy bonus from the loss (the larger the entropy the smaller the loss).
+        loss -= self.weight_entropy.get(time_percentage) * entropy
+        return loss
 
     @rlgraph_api
-    def _graph_fn_baseline_loss_per_item(self, baseline_values, advantages):
+    def _graph_fn_state_value_function_loss_per_item(self, state_values, advantages, time_percentage=None):
         """
         Computes the loss for V(s).
 
         Args:
-            baseline_values (SingleDataOp): Baseline predictions V(s).
+            state_values (SingleDataOp): Baseline predictions V(s).
             advantages (SingleDataOp): Advantage values.
 
         Returns:
@@ -152,13 +150,13 @@ class ActorCriticLossFunction(LossFunction):
         """
         v_targets = None
         if get_backend() == "tf":
-            baseline_values = tf.squeeze(input=baseline_values, axis=-1)
-            v_targets = advantages + baseline_values
+            state_values = tf.squeeze(input=state_values, axis=-1)
+            v_targets = advantages + state_values
             v_targets = tf.stop_gradient(input=v_targets)
         elif get_backend() == "pytorch":
-            baseline_values = torch.squeeze(baseline_values, dim=-1)
-            v_targets = advantages + baseline_values
+            state_values = torch.squeeze(state_values, dim=-1)
+            v_targets = advantages + state_values
             v_targets = v_targets.detach()
 
-        baseline_loss = (v_targets - baseline_values) ** 2
-        return baseline_loss
+        vf_loss = (v_targets - state_values) ** 2
+        return self.weight_vf.get(time_percentage) * vf_loss
