@@ -49,7 +49,6 @@ class PrioritizedReplay(Memory):
 
         # Variables.
         self.index = None
-        self.size = None
         self.max_priority = None
         self.sum_segment_buffer = None
         self.sum_segment_tree = None
@@ -75,8 +74,6 @@ class PrioritizedReplay(Memory):
 
         # Main buffer index.
         self.index = self.get_variable(name="index", dtype=int, trainable=False, initializer=0)
-        # Number of elements present.
-        self.size = self.get_variable(name="size", dtype=int, trainable=False, initializer=0)
 
         self.max_priority = self.get_variable(name="max-priority", dtype=float, trainable=False, initializer=1.0)
 
@@ -108,15 +105,15 @@ class PrioritizedReplay(Memory):
 
     @rlgraph_api(flatten_ops=True)
     def _graph_fn_insert_records(self, records):
-        num_records = get_batch_size(records["terminals"])
+        num_records = get_batch_size(records[self.terminal_key])
         index = self.read_variable(self.index)
         update_indices = tf.range(start=index, limit=index + num_records) % self.capacity
 
         # Updates all the necessary sub-variables in the record.
         record_updates = list()
-        for key in self.record_registry:
+        for key in self.memory:
             record_updates.append(self.scatter_update_variable(
-                variable=self.record_registry[key],
+                variable=self.memory[key],
                 indices=update_indices,
                 updates=records[key]
             ))
