@@ -17,11 +17,13 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import re
+
 from rlgraph import get_backend
 from rlgraph.components.layers.preprocessing import PreprocessLayer
-from rlgraph.utils.util import default_dict
 from rlgraph.components.neural_networks.stack import Stack
 from rlgraph.utils.decorators import rlgraph_api, graph_fn
+from rlgraph.utils.util import default_dict
 
 if get_backend() == "tf":
     import tensorflow as tf
@@ -51,9 +53,9 @@ class PreprocessorStack(Stack):
         """
         self.fold_time_rank = kwargs.get("fold_time_rank", False)
         self.unfold_time_rank = kwargs.get("unfold_time_rank", False)
-        # Link sub-Components' `apply` methods together to yield PreprocessorStack's `preprocess` method.
+        # Link sub-Components' `call` methods together to yield PreprocessorStack's `preprocess` method.
         # NOTE: Do not include `reset` here as it is defined explicitly below.
-        kwargs["api_methods"] = [dict(api="preprocess", component_api="apply", fold_time_rank=self.fold_time_rank,
+        kwargs["api_methods"] = [dict(api="preprocess", component_api="call", fold_time_rank=self.fold_time_rank,
                                       unfold_time_rank=self.unfold_time_rank)]
         default_dict(kwargs, dict(scope=kwargs.pop("scope", "preprocessor-stack")))
         super(PreprocessorStack, self).__init__(*preprocessors, **kwargs)
@@ -63,7 +65,7 @@ class PreprocessorStack(Stack):
         # TODO: python-Components: For now, we call each preprocessor's graph_fn directly.
         if self.backend == "python" or get_backend() == "python":
             for preprocess_layer in self.sub_components.values():  # type: PreprocessLayer
-                if preprocess_layer.scope in ["time-rank-folder_", "time-rank-unfolder_"]:
+                if re.search(r'^\.helper-', preprocess_layer.scope):
                     continue
                 preprocess_layer._graph_fn_reset()
 
@@ -71,7 +73,7 @@ class PreprocessorStack(Stack):
             # Connect each pre-processor's "reset" output op via our graph_fn into one op.
             resets = list()
             for preprocess_layer in self.sub_components.values():  # type: PreprocessLayer
-                if preprocess_layer.scope in ["time-rank-folder_", "time-rank-unfolder_"]:
+                if re.search(r'^\.helper-', preprocess_layer.scope):
                     continue
                 resets.append(preprocess_layer.reset())
             reset_op = self._graph_fn_reset(*resets)
@@ -94,8 +96,9 @@ class PreprocessorStack(Stack):
             Space: The Space after preprocessing.
         """
         for pp in self.sub_components.values():
-            if pp.scope not in ["time-rank-folder_", "time-rank-unfolder_"] or \
-                    (pp.scope == "time-rank-folder_" and self.fold_time_rank) or \
-                    (pp.scope == "time-rank-unfolder_" and self.unfold_time_rank):
+            # not in ["time-rank-folder_", "time-rank-unfolder_"] or \
+            if not re.search(r'\.helper-', pp.scope) or \
+                    (pp.scope == ".helper-time-rank-folder" and self.fold_time_rank) or \
+                    (pp.scope == ".helper-time-rank-unfolder" and self.unfold_time_rank):
                 space = pp.get_preprocessed_space(space)
         return space
