@@ -18,6 +18,7 @@ from __future__ import division
 from __future__ import print_function
 
 import numpy as np
+from rlgraph.utils import SMALL_NUMBER
 from six.moves import xrange as range_
 from rlgraph import get_distributed_backend
 from rlgraph.execution.ray.apex.apex_memory import ApexMemory
@@ -80,21 +81,20 @@ class RayMemoryActor(RayActor):
         else:
             rewards = records["rewards"]
         for i in range_(num_records):
+            # If Actions is dict with vectors per key, convert to single dict.
+            if isinstance(records["actions"], dict):
+                action = {k: v[i] for k, v in records["actions"].items()}
+            else:
+                action = records["actions"][i]
+
             self.memory.insert_records((
                 records["states"][i],
-                records["actions"][i],
+                action,
                 rewards[i],
                 records["terminals"][i],
                 records["next_states"][i],
                 records["importance_weights"][i]
             ))
-            # self.memory.insert_records((
-            #     records["states"][i],
-            #     records["actions"][i],
-            #     rewards[i],
-            #     records["terminals"][i],
-            #     records["importance_weights"][i]
-            # ))
 
     def update_priorities(self, indices, loss):
         """
@@ -105,4 +105,5 @@ class RayMemoryActor(RayActor):
             indices (ndarray): Indices to update in replay memory.
             loss (ndarray):  Loss values for indices.
         """
-        self.memory.update_records(indices, loss)
\ No newline at end of file
+        loss = np.abs(loss) + SMALL_NUMBER
+        self.memory.update_records(indices, loss)
