@@ -83,6 +83,10 @@ class RayPolicyWorker(RayActor):
         self.agent = self.setup_agent(agent_config, worker_spec)
         self.worker_frameskip = frameskip
 
+        #  Flag for container actions.
+        self.container_actions = self.agent.flat_action_space is not None
+        self.action_space = self.agent.flat_action_space
+
         # Save these so they can be fetched after training if desired.
         self.finished_episode_rewards = [[] for _ in range_(self.num_environments)]
         self.finished_episode_timesteps = [[] for _ in range_(self.num_environments)]
@@ -127,28 +131,6 @@ class RayPolicyWorker(RayActor):
     def as_remote(cls, num_cpus=None, num_gpus=None):
         return ray.remote(num_cpus=num_cpus, num_gpus=num_gpus)(cls)
 
-    def setup_preprocessor(self, preprocessing_spec, in_space):
-        if preprocessing_spec is not None:
-            # TODO move ingraph for python component assembly.
-            preprocessing_spec = deepcopy(preprocessing_spec)
-            in_space = deepcopy(in_space)
-            # Set scopes.
-            scopes = [preprocessor["scope"] for preprocessor in preprocessing_spec]
-            # Set backend to python.
-            for spec in preprocessing_spec:
-                spec["backend"] = "python"
-            processor_stack = PreprocessorStack(*preprocessing_spec, backend="python")
-            build_space = in_space
-            for sub_comp_scope in scopes:
-                processor_stack.sub_components[sub_comp_scope].create_variables(input_spaces=dict(
-                    preprocessing_inputs=build_space
-                ), action_space=None)
-                build_space = processor_stack.sub_components[sub_comp_scope].get_preprocessed_space(build_space)
-            processor_stack.reset()
-            return processor_stack
-        else:
-            return None
-
     def setup_agent(self, agent_config, worker_spec):
         """
         Sets up agent, potentially modifying its configuration via worker specific settings.
@@ -188,7 +170,11 @@ class RayPolicyWorker(RayActor):
         env_frames = 0
 
         # Final result batch.
-        batch_states, batch_actions, batch_rewards, batch_terminals = [], [], [], []
+        if self.container_actions:
+            batch_actions = {k: [] for k in self.action_space.keys()}
+        else:
+            batch_actions = []
+        batch_states, batch_rewards, batch_next_states, batch_terminals = [], [], [], []
 
         # Running trajectories.
         sample_states, sample_actions, sample_rewards, sample_terminals = {}, {}, {}, {}
@@ -197,11 +183,15 @@ class RayPolicyWorker(RayActor):
         # from previous execution was terminal for that environment.
         for i, env_id in enumerate(self.env_ids):
             sample_states[env_id] = []
-            sample_actions[env_id] = []
+            if self.container_actions:
+                sample_actions[env_id] = {k: [] for k in self.action_space.keys()}
+            else:
+                sample_actions[env_id] = []
             sample_rewards[env_id] = []
             sample_terminals[env_id] = []
 
         env_states = self.last_states
+        last_episode_rewards = []
         current_episode_rewards = self.last_ep_rewards
         current_episode_timesteps = self.last_ep_timesteps
         current_episode_start_timestamps = self.last_ep_start_timestamps
@@ -221,10 +211,25 @@ class RayPolicyWorker(RayActor):
                 else:
                     self.preprocessed_states_buffer[i] = env_states[i]
 
-            actions = self.get_action(states=self.preprocessed_states_buffer,
-                                      use_exploration=use_exploration, apply_preprocessing=False)
+            actions = self.agent.get_action(states=self.preprocessed_states_buffer,
+                                            use_exploration=use_exploration, apply_preprocessing=False)
 
-            next_states, step_rewards, terminals, infos = self.vector_env.step(actions=actions)
+            if self.agent.flat_action_space is not None:
+                some_key = next(iter(actions))
+                assert isinstance(actions, dict) and isinstance(actions[some_key], np.ndarray),\
+                    "ERROR: Cannot flip container-action batch with dict keys if returned value is not a dict OR " \
+                    "values of returned value are not np.ndarrays!"
+                if hasattr(actions[some_key], "__len__"):
+                    env_actions = [{key: value[i] for key, value in actions.items()} for i in range(len(actions[some_key]))]
+                else:
+                    # Action was not array type.
+                    env_actions = actions
+            # No flipping necessary.
+            else:
+                env_actions = actions
+                if self.num_environments == 1 and env_actions.shape == ():
+                    env_actions = [env_actions]
+            next_states, step_rewards, terminals, infos = self.vector_env.step(actions=env_actions)
             # Worker frameskip not needed as done in env.
             # for _ in range_(self.worker_frameskip):
             #     next_states, step_rewards, terminals, infos = self.vector_env.step(actions=actions)
@@ -249,7 +254,11 @@ class RayPolicyWorker(RayActor):
                 # Each position is the running episode reward of that episode. Add step reward.
                 current_episode_rewards[i] += step_rewards[i]
                 sample_states[env_id].append(state_buffer[i])
-                sample_actions[env_id].append(actions[i])
+                if self.container_actions:
+                    for name in self.action_space.keys():
+                        sample_actions[env_id][name].append(env_actions[i][name])
+                else:
+                    sample_actions[env_id].append(env_actions[i])
                 sample_rewards[env_id].append(step_rewards[i])
                 sample_terminals[env_id].append(terminals[i])
                 current_episode_sample_times[i] += current_iteration_time
@@ -257,21 +266,31 @@ class RayPolicyWorker(RayActor):
                 # Terminate and reset episode for that environment.
                 if terminals[i] or (0 < max_timesteps_per_episode <= current_episode_timesteps[i]):
                     self.finished_episode_rewards[i].append(current_episode_rewards[i])
+
                     self.finished_episode_timesteps[i].append(current_episode_timesteps[i])
                     self.finished_episode_total_times[i].append(time.perf_counter() - current_episode_start_timestamps[i])
                     self.finished_episode_sample_times[i].append(current_episode_sample_times[i])
                     episodes_executed[i] += 1
                     self.episodes_executed += 1
+                    last_episode_rewards.append(current_episode_rewards[i])
 
                     # Append to final result trajectories.
                     batch_states.extend(sample_states[env_id])
-                    batch_actions.extend(sample_actions[env_id])
+                    if self.agent.flat_action_space is not None:
+                        # Use actions here, not env actions.
+                        for name in self.agent.flat_action_space.keys():
+                            batch_actions[name].extend(sample_actions[env_id][name])
+                    else:
+                        batch_actions.extend(sample_actions[env_id])
                     batch_rewards.extend(sample_rewards[env_id])
                     batch_terminals.extend(sample_terminals[env_id])
 
                     # Reset running trajectory for this env.
                     sample_states[env_id] = []
-                    sample_actions[env_id] = []
+                    if self.container_actions:
+                        sample_actions[env_id] = {k: [] for k in self.action_space.keys()}
+                    else:
+                        sample_actions[env_id] = []
                     sample_rewards[env_id] = []
                     sample_terminals[env_id] = []
 
@@ -309,7 +328,12 @@ class RayPolicyWorker(RayActor):
             # This env was not terminal -> need to process remaining trajectory
             if not terminals[i]:
                 batch_states.extend(sample_states[env_id])
-                batch_actions.extend(sample_actions[env_id])
+                if self.agent.flat_action_space is not None:
+                    # Use actions here, not env actions.
+                    for name in self.agent.flat_action_space.keys():
+                        batch_actions[name].extend(sample_actions[env_id][name])
+                else:
+                    batch_actions.extend(sample_actions[env_id])
                 batch_rewards.extend(sample_rewards[env_id])
                 batch_terminals.extend(sample_terminals[env_id])
 
@@ -334,6 +358,7 @@ class RayPolicyWorker(RayActor):
             sample_batch=sample_batch,
             batch_size=len(batch_rewards),
             metrics=dict(
+                last_rewards=last_episode_rewards,
                 runtime=total_time,
                 # Agent act/observe throughput.
                 timesteps_executed=timesteps_executed,
@@ -363,7 +388,7 @@ class RayPolicyWorker(RayActor):
         # Adjust env frames for internal env frameskip:
         adjusted_frames = [env_frames * self.env_frame_skip for env_frames in self.sample_env_frames]
         if len(self.finished_episode_rewards) > 0:
-            all_finished_rewards = list()
+            all_finished_rewards = []
             for env_reward_list in self.finished_episode_rewards:
                 all_finished_rewards.extend(env_reward_list)
             min_episode_reward = np.min(all_finished_rewards)
@@ -418,7 +443,3 @@ class RayPolicyWorker(RayActor):
             terminals=terminals
         ), len(rewards)
 
-    def get_action(self, states, use_exploration, apply_preprocessing):
-        action = self.agent.get_action(states=states, use_exploration=use_exploration,
-                                       apply_preprocessing=apply_preprocessing)
-        return [action] if action.shape == () else action
