@@ -115,10 +115,18 @@ class SyncBatchExecutor(RayExecutor):
 
         env_steps += num_samples
         # 3. Merge samples
+        rewards = []
+        for sample in sample_batches:
+            if len(sample.metrics["last_rewards"]) > 0:
+                rewards.extend(sample.metrics["last_rewards"])
         batch = merge_samples(sample_batches, decompress=self.compress_states)
 
         # 4. Update from merged batch.
         self.local_agent.update(batch, apply_postprocessing=False)
-        return env_steps, 1, 0, 0
+        return env_steps, 1, {
+            "discarded": 0,
+            "queue_inserts": 0,
+            "rewards": rewards
+        }
 
 
