@@ -47,7 +47,6 @@ class TensorFlowExecutor(GraphExecutor):
     """
     def __init__(self, **kwargs):
         super(TensorFlowExecutor, self).__init__(**kwargs)
-        self.global_training_timestep = None
         self.session_config = self.execution_spec["session_config"]
 
         # The tf.Graph object to be run in a tf session.
@@ -80,8 +79,7 @@ class TensorFlowExecutor(GraphExecutor):
         # The optimizer is a somewhat privileged graph component because it must manage
         # devices depending on the device strategy and we hence keep an instance here to be able
         # to request special device init ops.
-        self.optimizer = None
-        self.vf_optimizer = None
+        self.optimizers = None
 
         self.graph_default_context = None
         self.local_device_protos = device_lib.list_local_devices()
@@ -282,8 +280,8 @@ class TensorFlowExecutor(GraphExecutor):
 
     def init_execution(self):
         """
-        Creates and stores a tf server (and optionally joins it if we are a parameter-server).
-        Only relevant, if we are running in distributed mode.
+        Creates and sets up the distributed backend.
+        Also creates the global time step variable.
         """
         if self.execution_mode == "distributed":
             if get_distributed_backend() == "distributed_tf":
@@ -341,10 +339,18 @@ class TensorFlowExecutor(GraphExecutor):
             return assignments
 
     def setup_graph(self):
-        # Generate the tf-Graph object and enter its scope as default graph.
+        """
+        Generates the tf-Graph object and enters its scope as default graph.
+        Also creates the global time step variable.
+        """
         self.graph = tf.Graph()
         self.graph_default_context = self.graph.as_default()
         self.graph_default_context.__enter__()
+
+        self.global_training_timestep = tf.get_variable(
+            name="global-timestep", dtype=util.convert_dtype("int"), trainable=False, initializer=0,
+            collections=["global-timestep", tf.GraphKeys.GLOBAL_STEP])
+
         # Set the random seed graph-wide.
         if self.seed is not None:
             self.logger.info("Initializing TensorFlow graph with seed {}".format(self.seed))
@@ -370,7 +376,7 @@ class TensorFlowExecutor(GraphExecutor):
             hooks (list): List of hooks to use for Saver and Summarizer in Session. Should be appended to.
         """
         self.saver = tf.train.Saver(
-           var_list=list(self.graph_builder.root_component.variables.values()),
+           var_list=list(self.graph_builder.root_component.variable_registry.values()),
            reshape=False,
            sharded=False,
            max_to_keep=self.saver_spec.get("max_checkpoints", 1) if self.saver_spec else None,
@@ -442,20 +448,14 @@ class TensorFlowExecutor(GraphExecutor):
         Assigns the scaffold object to `self.scaffold`.
         """
         # Determine init_op and ready_op.
-        var_list = list(self.graph_builder.root_component.variables.values())
-
-        self.global_training_timestep = tf.get_variable(
-            name="global-timestep", dtype=util.convert_dtype("int"), trainable=False, initializer=0,
-            collections=["global-timestep", tf.GraphKeys.GLOBAL_STEP])
+        var_list = list(self.graph_builder.root_component.variable_registry.values())
         var_list.append(self.global_training_timestep)
 
         # We can not fetch optimizer vars.
         # TODO let graph builder do this
-        if self.optimizer is not None:
-            var_list.extend(self.optimizer.get_optimizer_variables())
-            # If the VF has a separate optimizer (non-shared network), we need to fetch its vars here as well.
-            if self.vf_optimizer is not None:
-                var_list.extend(self.vf_optimizer.get_optimizer_variables())
+        if self.optimizers is not None:
+            for optimizer in self.optimizers:
+                var_list.extend(optimizer.get_optimizer_variables())
 
         if self.execution_mode == "single":
             self.init_op = tf.variables_initializer(var_list=var_list)
@@ -567,10 +567,12 @@ class TensorFlowExecutor(GraphExecutor):
         else:
             # If monitoring is disabled,
             if self.disable_monitoring:
-                self.logger.info("Setting up default session for non-distributed mode.")
+                self.logger.info("Setting up default session for non-distributed mode. Session config: {}".format(
+                    self.tf_session_config))
                 self.monitored_session = tf.Session(config=self.tf_session_config)
             else:
-                self.logger.info("Setting up singular monitored session for non-distributed mode.")
+                self.logger.info("Setting up singular monitored session for non-distributed mode. Session config: {}".
+                                 format(self.tf_session_config))
                 self.monitored_session = tf.train.SingularMonitoredSession(
                     hooks=hooks,
                     scaffold=self.scaffold,
@@ -674,10 +676,14 @@ class TensorFlowExecutor(GraphExecutor):
             batch_size (int): The batch size that needs to be split between the different GPUs.
             extra_build_args (Optional[dict]): Extra build elements to pass.
         """
-        self.optimizer = root_optimizer
+        self.optimizers = []
+        if root_optimizer is not None:
+            self.optimizers.append(root_optimizer)
         # Save separate vf optimizer if necessary.
         if extra_build_args is not None and "vf_optimizer" in extra_build_args:
-            self.vf_optimizer = extra_build_args["vf_optimizer"]
+            self.optimizers.append(extra_build_args["vf_optimizer"])
+        if extra_build_args is not None and "optimizers" in extra_build_args:
+            self.optimizers.extend(extra_build_args["optimizers"])
 
         if self.device_strategy == "multi_gpu_sync":
             assert self.num_gpus > 1 or (self.fake_gpus is True and self.max_usable_gpus > 0), \
