@@ -23,7 +23,7 @@ import numpy as np
 from six.moves import xrange as range_
 
 from rlgraph import get_backend
-from rlgraph.spaces import Space
+from rlgraph.spaces.space import Space
 from rlgraph.utils.initializer import Initializer
 from rlgraph.utils.util import convert_dtype
 
@@ -37,7 +37,6 @@ class BoxSpace(Space):
     """
     A box in R^n with a shape tuple of len n. Each dimension may be bounded.
     """
-
     def __init__(self, low, high, shape=None, add_batch_rank=False, add_time_rank=False, time_major=False,
                  dtype=np.float32):
         """
@@ -77,7 +76,12 @@ class BoxSpace(Space):
         # to all values in all dimensions.
         # 0D Space.
         if self._shape == ():
-            assert isinstance(low, (int, float, bool))
+            if isinstance(low, np.ndarray):
+                assert low.shape == (), "ERROR: If shape == (), `low` must be scalar!"
+                low = np.asscalar(low)
+            if isinstance(high, np.ndarray):
+                assert high.shape == (), "ERROR: If shape == (), `high` must be scalar!"
+                high = np.asscalar(high)
             self.global_bounds = (low, high)
         # nD Space (n > 0). Bounds can be single number or individual bounds.
         else:
@@ -219,6 +223,9 @@ class BoxSpace(Space):
             return False
         return (sample >= self.low).all() and (sample <= self.high).all()
 
+    def map(self, mapping):
+        return mapping("", self)
+
     def __repr__(self):
         return "{}({} {} {}{})".format(
             type(self).__name__.title(), self.shape, str(self.dtype), "; +batch" if self.has_batch_rank else
@@ -231,6 +238,6 @@ class BoxSpace(Space):
                # np.allclose(self.low, other.low) and np.allclose(self.high, other.high) and \
 
     def __hash__(self):
-        if self.shape == ():
-            return hash((self.low, self.high))
+        if self.shape == () or self.global_bounds is not False:
+            return hash((np.asscalar(self.low), np.asscalar(self.high)))
         return hash((tuple(self.low), tuple(self.high)))
