@@ -29,6 +29,7 @@ from rlgraph.execution import SingleThreadedWorker
 from rlgraph.spaces import FloatBox, IntBox
 from rlgraph.tests.test_util import config_from_path, recursive_assert_almost_equal
 from rlgraph.utils import root_logger
+from rlgraph.utils.numpy import one_hot
 
 
 class TestDQNAgentShortTaskLearning(unittest.TestCase):
@@ -56,11 +57,10 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             action_space=dummy_env.action_space,
             execution_spec=dict(seed=12),
             update_spec=dict(update_interval=4, batch_size=24, sync_interval=32),
-            optimizer_spec=dict(type="adam", learning_rate=0.05),
-            store_last_q_table=True
+            optimizer_spec=dict(type="adam", learning_rate=0.05)
         )
 
-        time_steps = 1000
+        time_steps = 2000
         worker = SingleThreadedWorker(
             env_spec=lambda: GridWorld("2x2"),
             agent=agent,
@@ -69,25 +69,16 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
         )
         results = worker.execute_timesteps(time_steps, use_exploration=True)
 
-        print("STATES:\n{}".format(agent.last_q_table["states"]))
-        print("\n\nQ(s,a)-VALUES:\n{}".format(np.round_(agent.last_q_table["q_values"], decimals=2)))
-
         self.assertEqual(results["timesteps_executed"], time_steps)
         self.assertEqual(results["env_frames"], time_steps)
-        self.assertGreaterEqual(results["mean_episode_reward"], -3.5)
+        self.assertGreaterEqual(results["mean_episode_reward"], -1.5)
         self.assertGreaterEqual(results["max_episode_reward"], 0.0)
         self.assertLessEqual(results["episodes_executed"], time_steps / 2)
 
-        # Check q-table for correct values.
-        expected_q_values_per_state = {
-            (1.0, 0, 0, 0): (-1, -5, 0, -1),
-            (0, 1.0, 0, 0): (-1, 1, 0, 0)
-        }
-        for state, q_values in zip(agent.last_q_table["states"], agent.last_q_table["q_values"]):
-            state, q_values = tuple(state), tuple(q_values)
-            assert state in expected_q_values_per_state, \
-                "ERROR: state '{}' not expected in q-table as it's a terminal state!".format(state)
-            recursive_assert_almost_equal(q_values, expected_q_values_per_state[state], decimals=0)
+        # Check all learnt Q-values.
+        q_values = agent.graph_executor.execute(("get_q_values", one_hot(np.array([0, 1]), depth=4)))[:]
+        recursive_assert_almost_equal(q_values[0], (0.8, -5, 0.9, 0.8), decimals=1)
+        recursive_assert_almost_equal(q_values[1], (0.8, 1.0, 0.9, 0.9), decimals=1)
 
     def test_double_dqn_on_2x2_grid_world(self):
         """
@@ -104,11 +95,10 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             action_space=dummy_env.action_space,
             execution_spec=dict(seed=10),
             update_spec=dict(update_interval=4, batch_size=24, sync_interval=32),
-            optimizer_spec=dict(type="adam", learning_rate=0.05),
-            store_last_q_table=True
+            optimizer_spec=dict(type="adam", learning_rate=0.05)
         )
 
-        time_steps = 1000
+        time_steps = 2000
         worker = SingleThreadedWorker(
             env_spec=lambda: GridWorld.from_spec(env_spec),
             agent=agent,
@@ -117,25 +107,16 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
         )
         results = worker.execute_timesteps(time_steps, use_exploration=True)
 
-        print("STATES:\n{}".format(agent.last_q_table["states"]))
-        print("\n\nQ(s,a)-VALUES:\n{}".format(np.round_(agent.last_q_table["q_values"], decimals=2)))
-
         self.assertEqual(results["timesteps_executed"], time_steps)
         self.assertEqual(results["env_frames"], time_steps)
-        self.assertGreaterEqual(results["mean_episode_reward"], -4.5)
+        self.assertGreaterEqual(results["mean_episode_reward"], -1.5)
         self.assertGreaterEqual(results["max_episode_reward"], 0.0)
-        self.assertLessEqual(results["episodes_executed"], 350)
+        self.assertLessEqual(results["episodes_executed"], time_steps / 2)
 
-        # Check q-table for correct values.
-        expected_q_values_per_state = {
-            (1.0, 0, 0, 0): (-1, -5, 0, -1),
-            (0, 1.0, 0, 0): (-1, 1, 0, 0)
-        }
-        for state, q_values in zip(agent.last_q_table["states"], agent.last_q_table["q_values"]):
-            state, q_values = tuple(state), tuple(q_values)
-            assert state in expected_q_values_per_state, \
-                "ERROR: state '{}' not expected in q-table as it's a terminal state!".format(state)
-            recursive_assert_almost_equal(q_values, expected_q_values_per_state[state], decimals=0)
+        # Check all learnt Q-values.
+        q_values = agent.graph_executor.execute(("get_q_values", one_hot(np.array([0, 1]), depth=4)))[:]
+        recursive_assert_almost_equal(q_values[0], (0.8, -5, 0.9, 0.8), decimals=1)
+        recursive_assert_almost_equal(q_values[1], (0.8, 1.0, 0.9, 0.9), decimals=1)
 
     def test_double_dqn_on_2x2_grid_world_with_container_actions(self):
         """
@@ -153,8 +134,7 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             dueling_q=False,
             state_space=FloatBox(shape=(4,)),
             action_space=dummy_env.action_space,
-            execution_spec=dict(seed=15),
-            store_last_q_table=True
+            execution_spec=dict(seed=15)
         )
 
         time_steps = 10000
@@ -163,12 +143,12 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             agent=agent,
             preprocessing_spec=preprocessing_spec,
             worker_executes_preprocessing=True,
-            render=False
+            render=False,
+            episode_finish_callback=lambda episode_return, duration, timesteps, env_num:
+            print("episode return {}; steps={}".format(episode_return, timesteps))
         )
         results = worker.execute_timesteps(time_steps, use_exploration=True)
 
-        print("LAST q-table:\n{}".format(agent.last_q_table))
-
         self.assertEqual(results["timesteps_executed"], time_steps)
         self.assertEqual(results["env_frames"], time_steps)
         self.assertGreaterEqual(results["mean_episode_reward"], -2.0)
@@ -177,10 +157,10 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
 
         # Check q-table for correct values.
         expected_q_values_per_state = {
-            (0., 0., -1., 0.): {"forward": (-5.0, -1.0, -1.0), "jump": (0.0, -1.0)},
-            (0., 0., 1., 0.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},
-            (0., 0., 0., -1.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},
-            (0., 0., 0., 1.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},
+            (0., 0., -1., 0.): {"forward": (-5.0, -1.0, -1.0), "jump": (0.0, -1.0)},  # start <
+            (0., 0., 1., 0.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},    # start >
+            (0., 0., 0., -1.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},   # start v
+            (0., 0., 0., 1.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},    # start ^
             (0., 1., -1., 0.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},
             (0., 1., 1., 0.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},
             (0., 1., 0., -1.): {"forward": (0.0, -1.0, -1.0), "jump": (0.0, -1.0)},
@@ -208,13 +188,11 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             dueling_q=False,
             state_space=self.grid_world_4x4_flattened_state_space,
             action_space=dummy_env.action_space,
-            execution_spec=dict(seed=10),
             update_spec=dict(update_interval=4, batch_size=32, sync_interval=32),
-            optimizer_spec=dict(type="adam", learning_rate=0.02),
-            store_last_q_table=True
+            optimizer_spec=dict(type="adam", learning_rate=["linear", 0.01, 0.0001])
         )
 
-        time_steps = 6000
+        time_steps = 9000
         worker = SingleThreadedWorker(
             env_spec=lambda: GridWorld("4x4"),
             agent=agent,
@@ -223,39 +201,25 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
         )
         results = worker.execute_timesteps(time_steps, use_exploration=True)
 
-        print("STATES:\n{}".format(agent.last_q_table["states"]))
-        print("\n\nQ(s,a)-VALUES:\n{}".format(np.round_(agent.last_q_table["q_values"], decimals=2)))
-
         self.assertEqual(results["timesteps_executed"], time_steps)
         self.assertEqual(results["env_frames"], time_steps)
-        self.assertGreaterEqual(results["mean_episode_reward"], -10)
-        self.assertGreaterEqual(results["max_episode_reward"], -4)
+        self.assertGreaterEqual(results["mean_episode_reward"], -7.0)
+        self.assertGreaterEqual(results["max_episode_reward"], 0.0)
         self.assertLessEqual(results["episodes_executed"], time_steps / 5)
 
-        # Check q-table for correct values.
-        expected_q_values_per_state = {
-            (1.0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0): (-5, -4, -4, -4),  # 0
-            (0, 1.0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0): (-5, -5, -3, -4),  # 1
-            (0, 0, 1.0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0): (-4, -2, -5, -3),  # 2
-            # 3=terminal
-            (0, 0, 0, 0, 1.0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0): (-4, -3, -5, -5),  # 4
-            # 5=terminal
-            (0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0, 0, 0, 0, 0, 0, 0): (-5, -1, -1, -3),  # 6
-            (0, 0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0, 0, 0, 0, 0, 0): (-2, 0, -1, -5),  # 7
-            (0, 0, 0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0, 0, 0, 0, 0): (-3, -4, -2, -4),  # 8
-            (0, 0, 0, 0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0, 0, 0, 0): (-3, -5, -1, -5),  # 9
-            (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0, 0, 0): (-2, -5, 0, -2),  # 10
-            (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0, 0): (-1, 1, 0, -1),  # 11
-            (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1.0, 0, 0, 0): (-4, -4, -5, -3),  # 12
-            # 13=terminal
-            # 14=terminal
-            # 15=terminal
-        }
-        for state, q_values in zip(agent.last_q_table["states"], agent.last_q_table["q_values"]):
-            state, q_values = tuple(state), tuple(q_values)
-            assert state in expected_q_values_per_state, \
-                "ERROR: state '{}' not expected in q-table as it's a terminal state!".format(state)
-            recursive_assert_almost_equal(q_values, expected_q_values_per_state[state], decimals=0)
+        # Check all learnt Q-values.
+        q_values = agent.graph_executor.execute(
+            ("get_q_values", one_hot(np.array([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]), depth=16))
+        )
+        # Walking towards goal is always goo.
+        self.assertTrue(q_values[0][0] < q_values[0][1])
+        self.assertTrue(q_values[6][3] < q_values[6][1])
+        # Falling into holes.
+        self.assertTrue(q_values[1][1] < -4.0)  # Going right in state 1 is very bad
+        self.assertTrue(q_values[4][2] < -4.0)  # Going down in state 4 is very bad
+        self.assertTrue(q_values[12][2] < -4.0)  # Going down in state 12 is very bad
+        # Reaching goal.
+        self.assertTrue(q_values[11][1] > 0.0)
 
     def test_dqn_on_cart_pole(self):
         """
@@ -270,8 +234,7 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             action_space=dummy_env.action_space,
             execution_spec=dict(seed=15),
             update_spec=dict(update_interval=4, batch_size=24, sync_interval=64),
-            optimizer_spec=dict(type="adam", learning_rate=0.05),
-            store_last_q_table=True
+            optimizer_spec=dict(type="adam", learning_rate=0.05)
         )
 
         time_steps = 3000
@@ -308,11 +271,10 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             action_space=dummy_env.action_space,
             execution_spec=dict(seed=13),
             update_spec=dict(update_interval=4, batch_size=64, sync_interval=16),
-            optimizer_spec=dict(type="adam", learning_rate=0.01),
-            store_last_q_table=True
+            optimizer_spec=dict(type="adam", learning_rate=["linear", 0.01, 0.00001])
         )
 
-        time_steps = 3000
+        time_steps = 6000
         worker = SingleThreadedWorker(
             env_spec=lambda: OpenAIGymEnv(gym_env, seed=10),
             agent=agent,
@@ -323,8 +285,8 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
 
         self.assertEqual(results["timesteps_executed"], time_steps)
         self.assertEqual(results["env_frames"], time_steps)
-        self.assertGreaterEqual(results["mean_episode_reward"], 25)
-        self.assertLessEqual(results["episodes_executed"], 150)
+        self.assertGreaterEqual(results["mean_episode_reward"], 40)
+        self.assertLessEqual(results["episodes_executed"], time_steps / 10)
 
     def test_double_dqn_on_2x2_grid_world_single_action_to_container(self):
         """
@@ -343,8 +305,7 @@ class TestDQNAgentShortTaskLearning(unittest.TestCase):
             double_q=True,
             dueling_q=True,
             state_space=FloatBox(shape=(4,)),
-            action_space=action_space,
-            store_last_q_table=True
+            action_space=action_space
         )
 
         time_steps = 10000
