@@ -17,14 +17,15 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import numpy as np
 import unittest
 
+import numpy as np
+
 from rlgraph.components.policies import Policy, SharedValueFunctionPolicy, DuelingPolicy
 from rlgraph.spaces import *
 from rlgraph.tests import ComponentTest
-from rlgraph.tests.test_util import config_from_path
-from rlgraph.utils import softmax, relu
+from rlgraph.tests.test_util import config_from_path, recursive_assert_almost_equal
+from rlgraph.utils import softmax, relu, SMALL_NUMBER
 
 
 class TestPoliciesOnContainerActions(unittest.TestCase):
@@ -51,10 +52,8 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         test = ComponentTest(
             component=policy,
             input_spaces=dict(
-                nn_input=state_space,
+                nn_inputs=state_space,
                 actions=action_space,
-                #parameters=flat_float_action_space,
-                #logits=flat_float_action_space
             ),
             action_space=action_space
         )
@@ -64,24 +63,29 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         states = state_space.sample(2)
         # Raw NN-output.
         expected_nn_output = np.matmul(states, policy_params["policy/test-network/hidden-layer/dense/kernel"])
-        test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=6)
+        test.test(("get_nn_outputs", states), expected_outputs=expected_nn_output, decimals=6)
 
         # Raw action layers' output.
         expected_action_layer_outputs = dict(
             a=np.matmul(expected_nn_output, policy_params["policy/action-adapter-0/action-network/action-layer/dense/kernel"]),
             b=np.matmul(expected_nn_output, policy_params["policy/action-adapter-1/action-network/action-layer/dense/kernel"])
         )
-        test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_outputs),
-                  decimals=5)
+        test.test(
+            ("get_adapter_outputs", states),
+            expected_outputs=dict(adapter_outputs=expected_action_layer_outputs, nn_outputs=expected_nn_output),
+            decimals=5
+        )
 
         # Logits, parameters (probs) and skip log-probs (numerically unstable for small probs).
         expected_parameters_output = dict(
-            a=np.array(softmax(expected_action_layer_outputs["a"], axis=-1), dtype=np.float32),
-            b=np.array(softmax(expected_action_layer_outputs["b"], axis=-1), dtype=np.float32)
+            a=np.array(np.maximum(softmax(expected_action_layer_outputs["a"], axis=-1), SMALL_NUMBER), dtype=np.float32),
+            b=np.array(np.maximum(softmax(expected_action_layer_outputs["b"], axis=-1), SMALL_NUMBER), dtype=np.float32)
+        )
+        test.test(
+            ("get_adapter_outputs_and_parameters", states, ["adapter_outputs", "parameters"]),
+            expected_outputs=dict(adapter_outputs=expected_action_layer_outputs, parameters=expected_parameters_output),
+            decimals=5
         )
-        test.test(("get_logits_parameters_log_probs", states, ["logits", "parameters"]), expected_outputs=dict(
-            logits=expected_action_layer_outputs, parameters=expected_parameters_output
-        ), decimals=5)
 
         print("Probs: {}".format(expected_parameters_output))
 
@@ -91,6 +95,22 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         )
         test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
+        out = test.test(("get_action_and_log_likelihood", states))
+        action = out["action"]
+        llh = out["log_likelihood"]
+
+        # Action log-likelihood (sum of the composite llhs).
+        expected_action_llh_output = np.log(np.array([expected_parameters_output["a"][0][action["a"][0]],
+                                                      expected_parameters_output["a"][1][action["a"][1]]])) + \
+                                     np.log(np.array([expected_parameters_output["b"][0][action["b"][0]],
+                                                      expected_parameters_output["b"][1][action["b"][1]]]))
+        test.test(
+            ("get_log_likelihood", [states, action]), expected_outputs=dict(
+                log_likelihood=expected_action_llh_output, adapter_outputs=expected_action_layer_outputs
+            ), decimals=5
+        )
+        recursive_assert_almost_equal(expected_action_llh_output, llh, decimals=5)
+
         # Stochastic sample.
         out = test.test(("get_stochastic_action", states), expected_outputs=None)  # dict(action=expected_actions))
         self.assertTrue(out["action"]["a"].dtype == np.int32)
@@ -112,19 +132,6 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         self.assertTrue(out["entropy"]["b"].dtype == np.float32)
         self.assertTrue(out["entropy"]["b"].shape == (2,))
 
-        # Action log-probs.
-        expected_action_log_prob_output = dict(
-            a=np.log(np.array([expected_parameters_output["a"][0][expected_actions["a"][0]],
-                               expected_parameters_output["a"][1][expected_actions["a"][1]]])),
-            b=np.log(np.array([expected_parameters_output["b"][0][expected_actions["b"][0]],
-                               expected_parameters_output["b"][1][expected_actions["b"][1]]])),
-        )
-        test.test(
-            ("get_action_log_probs", [states, expected_actions]), expected_outputs=dict(
-                action_log_probs=expected_action_log_prob_output, logits=expected_action_layer_outputs
-            ), decimals=5
-        )
-
     def test_shared_value_function_policy_for_discrete_container_action_space(self):
         # state_space (NN is a simple single fc-layer relu network (2 units), random biases, random weights).
         state_space = FloatBox(shape=(5,), add_batch_rank=True)
@@ -136,12 +143,12 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
             b=Dict(b1=IntBox(3), b2=IntBox(4)),
             add_batch_rank=True
         )
-        flat_float_action_space = dict(
-            type="dict",
-            a=FloatBox(shape=(2,)),
-            b=Dict(b1=FloatBox(shape=(3,)), b2=FloatBox(shape=(4,))),
-            add_batch_rank=True
-        )
+        #flat_float_action_space = dict(
+        #    type="dict",
+        #    a=FloatBox(shape=(2,)),
+        #    b=Dict(b1=FloatBox(shape=(3,)), b2=FloatBox(shape=(4,))),
+        #    add_batch_rank=True
+        #)
 
         # Policy with baseline action adapter.
         shared_value_function_policy = SharedValueFunctionPolicy(
@@ -151,10 +158,8 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         test = ComponentTest(
             component=shared_value_function_policy,
             input_spaces=dict(
-                nn_input=state_space,
+                nn_inputs=state_space,
                 actions=action_space,
-                #parameters=flat_float_action_space,
-                #logits=flat_float_action_space
             ),
             action_space=action_space,
         )
@@ -168,7 +173,7 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         expected_nn_output = relu(np.matmul(
             states, policy_params["shared-value-function-policy/test-network/hidden-layer/dense/kernel"]
         ), 0.1)
-        test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=5)
+        test.test(("get_nn_outputs", states), expected_outputs=expected_nn_output, decimals=5)
 
         # Raw action layers' output.
         expected_action_layer_outputs = dict(
@@ -176,31 +181,41 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
             b=dict(b1=np.matmul(expected_nn_output, policy_params[base_scope + "1/action-network/action-layer/dense/kernel"]),
                    b2=np.matmul(expected_nn_output, policy_params[base_scope + "2/action-network/action-layer/dense/kernel"]))
         )
-        test.test(("get_action_layer_output", states), expected_outputs=dict(output=expected_action_layer_outputs),
-                  decimals=5)
+        test.test(
+            ("get_adapter_outputs", states),
+            expected_outputs=dict(adapter_outputs=expected_action_layer_outputs, nn_outputs=expected_nn_output),
+                decimals=5
+        )
 
         # State-values.
         expected_state_value_output = np.matmul(
             expected_nn_output, policy_params["shared-value-function-policy/value-function-node/dense-layer/dense/kernel"]
         )
-        test.test(("get_state_values", states), expected_outputs=dict(state_values=expected_state_value_output),
-                  decimals=5)
+        test.test(
+            ("get_state_values", states, ["state_values"]),
+            expected_outputs=dict(state_values=expected_state_value_output),
+            decimals=5
+        )
 
         # logits-values: One for each action-choice per item in the batch (simply take the remaining out nodes).
-        test.test(("get_state_values_logits_parameters_log_probs", states, ["state_values", "logits"]),
-                  expected_outputs=dict(state_values=expected_state_value_output, logits=expected_action_layer_outputs),
-                  decimals=5)
+        test.test(
+            ("get_state_values_adapter_outputs_and_parameters", states, ["state_values", "adapter_outputs"]),
+            expected_outputs=dict(
+                state_values=expected_state_value_output, adapter_outputs=expected_action_layer_outputs
+            ),
+            decimals=5
+        )
 
         # Parameter (probabilities). Softmaxed logits.
         expected_parameters_output = dict(
-            a=softmax(expected_action_layer_outputs["a"], axis=-1),
+            a=np.maximum(softmax(expected_action_layer_outputs["a"], axis=-1), SMALL_NUMBER),
             b=dict(
-                b1=softmax(expected_action_layer_outputs["b"]["b1"], axis=-1),
-                b2=softmax(expected_action_layer_outputs["b"]["b2"], axis=-1)
+                b1=np.maximum(softmax(expected_action_layer_outputs["b"]["b1"], axis=-1), SMALL_NUMBER),
+                b2=np.maximum(softmax(expected_action_layer_outputs["b"]["b2"], axis=-1), SMALL_NUMBER)
             )
         )
-        test.test(("get_logits_parameters_log_probs", states, ["logits", "parameters"]), expected_outputs=dict(
-            logits=expected_action_layer_outputs,
+        test.test(("get_adapter_outputs_and_parameters", states, ["adapter_outputs", "parameters"]), expected_outputs=dict(
+            adapter_outputs=expected_action_layer_outputs,
             parameters=expected_parameters_output
         ), decimals=5)
 
@@ -216,32 +231,54 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         )
         test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
+        out = test.test(("get_action_and_log_likelihood", states))
+        action = out["action"]
+        llh = out["log_likelihood"]
+
+        # Action log-likelihood.
+        expected_action_llh_output = np.log(np.array([expected_parameters_output["a"][0][action["a"][0]],
+                                                      expected_parameters_output["a"][1][action["a"][1]]])) + \
+                                     np.log(np.array([expected_parameters_output["b"]["b1"][0][action["b"]["b1"][0]],
+                                                      expected_parameters_output["b"]["b1"][1][action["b"]["b1"][1]]
+                                                      ])
+                                            ) + \
+                                     np.log(np.array([expected_parameters_output["b"]["b2"][0][action["b"]["b2"][0]],
+                                                      expected_parameters_output["b"]["b2"][1][action["b"]["b2"][1]],
+                                                      ])
+                                            )
+        test.test(
+            ("get_log_likelihood", [states, action]), expected_outputs=dict(
+                log_likelihood=expected_action_llh_output, adapter_outputs=expected_action_layer_outputs
+            ), decimals=5
+        )
+        recursive_assert_almost_equal(expected_action_llh_output, llh, decimals=5)
+
         # Stochastic sample.
-        out = test.test(("get_stochastic_action", states), expected_outputs=None)
-        self.assertTrue(out["action"]["a"].dtype == np.int32)
-        self.assertTrue(out["action"]["a"].shape == (2,))
-        self.assertTrue(out["action"]["b"]["b1"].dtype == np.int32)
-        self.assertTrue(out["action"]["b"]["b1"].shape == (2,))
-        self.assertTrue(out["action"]["b"]["b2"].dtype == np.int32)
-        self.assertTrue(out["action"]["b"]["b2"].shape == (2,))
+        out = test.test(("get_stochastic_action", states), expected_outputs=None)["action"]
+        self.assertTrue(out["a"].dtype == np.int32)
+        self.assertTrue(out["a"].shape == (2,))
+        self.assertTrue(out["b"]["b1"].dtype == np.int32)
+        self.assertTrue(out["b"]["b1"].shape == (2,))
+        self.assertTrue(out["b"]["b2"].dtype == np.int32)
+        self.assertTrue(out["b"]["b2"].shape == (2,))
 
         # Deterministic sample.
-        out = test.test(("get_deterministic_action", states), expected_outputs=None)
-        self.assertTrue(out["action"]["a"].dtype == np.int32)
-        self.assertTrue(out["action"]["a"].shape == (2,))
-        self.assertTrue(out["action"]["b"]["b1"].dtype == np.int32)
-        self.assertTrue(out["action"]["b"]["b1"].shape == (2,))
-        self.assertTrue(out["action"]["b"]["b2"].dtype == np.int32)
-        self.assertTrue(out["action"]["b"]["b2"].shape == (2,))
+        out = test.test(("get_deterministic_action", states), expected_outputs=None)["action"]
+        self.assertTrue(out["a"].dtype == np.int32)
+        self.assertTrue(out["a"].shape == (2,))
+        self.assertTrue(out["b"]["b1"].dtype == np.int32)
+        self.assertTrue(out["b"]["b1"].shape == (2,))
+        self.assertTrue(out["b"]["b2"].dtype == np.int32)
+        self.assertTrue(out["b"]["b2"].shape == (2,))
 
         # Distribution's entropy.
-        out = test.test(("get_entropy", states), expected_outputs=None)
-        self.assertTrue(out["entropy"]["a"].dtype == np.float32)
-        self.assertTrue(out["entropy"]["a"].shape == (2,))
-        self.assertTrue(out["entropy"]["b"]["b1"].dtype == np.float32)
-        self.assertTrue(out["entropy"]["b"]["b1"].shape == (2,))
-        self.assertTrue(out["entropy"]["b"]["b2"].dtype == np.float32)
-        self.assertTrue(out["entropy"]["b"]["b2"].shape == (2,))
+        out = test.test(("get_entropy", states), expected_outputs=None)["entropy"]
+        self.assertTrue(out["a"].dtype == np.float32)
+        self.assertTrue(out["a"].shape == (2,))
+        self.assertTrue(out["b"]["b1"].dtype == np.float32)
+        self.assertTrue(out["b"]["b1"].shape == (2,))
+        self.assertTrue(out["b"]["b2"].dtype == np.float32)
+        self.assertTrue(out["b"]["b2"].shape == (2,))
 
     def test_shared_value_function_policy_for_discrete_container_action_space_with_time_rank_folding(self):
         # state_space (NN is a simple single fc-layer relu network (2 units), random biases, random weights).
@@ -257,32 +294,32 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
             add_batch_rank=True,
             add_time_rank=True
         )
-        flat_float_action_space = Tuple(
-            FloatBox(shape=(2,)),
-            FloatBox(shape=(3,)),
-            Dict(
-                a=FloatBox(shape=(4,)),
-            ),
-            add_batch_rank=True,
-            add_time_rank=True
-        )
+        #flat_float_action_space = Tuple(
+        #    FloatBox(shape=(2,)),
+        #    FloatBox(shape=(3,)),
+        #    Dict(
+        #        a=FloatBox(shape=(4,)),
+        #    ),
+        #    add_batch_rank=True,
+        #    add_time_rank=True
+        #)
 
         # Policy with baseline action adapter AND batch-apply over the entire policy (NN + ActionAdapter + distr.).
         network_spec = config_from_path("configs/test_lrelu_nn.json")
         network_spec["fold_time_rank"] = True
+        network_spec["unfold_time_rank"] = True
         shared_value_function_policy = SharedValueFunctionPolicy(
             network_spec=network_spec,
-            action_adapter_spec=dict(unfold_time_rank=True),
+            action_adapter_spec=dict(fold_time_rank=True, unfold_time_rank=True),
             action_space=action_space,
+            value_fold_time_rank=True,
             value_unfold_time_rank=True
         )
         test = ComponentTest(
             component=shared_value_function_policy,
             input_spaces=dict(
-                nn_input=state_space,
+                nn_inputs=state_space,
                 actions=action_space,
-                #parameters=flat_float_action_space,
-                #logits=flat_float_action_space
             ),
             action_space=action_space,
         )
@@ -293,10 +330,10 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         states = state_space.sample(size=(2, 3))
         states_folded = np.reshape(states, newshape=(6, 6))
         # Raw NN-output (still folded).
-        expected_nn_output = relu(np.matmul(
+        expected_nn_output = np.reshape(relu(np.matmul(
             states_folded, policy_params["shared-value-function-policy/test-network/hidden-layer/dense/kernel"]
-        ), 0.1)
-        test.test(("get_nn_output", states), expected_outputs=dict(output=expected_nn_output), decimals=5)
+        ), 0.1), newshape=(2, 3, 3))
+        test.test(("get_nn_outputs", states), expected_outputs=expected_nn_output, decimals=5)
 
         # Raw action layer output; Expected shape=(3,3): 3=batch, 2=action categories + 1 state value
         expected_action_layer_output = tuple([
@@ -313,9 +350,11 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
                 a=np.reshape(expected_action_layer_output[2]["a"], newshape=(2, 3, 4))
             )
         ])
-        test.test(("get_action_layer_output", states),
-                  expected_outputs=dict(output=expected_action_layer_output_unfolded),
-                  decimals=5)
+        test.test(
+            ("get_adapter_outputs", states),
+            expected_outputs=dict(adapter_outputs=expected_action_layer_output_unfolded, nn_outputs=expected_nn_output),
+            decimals=5
+        )
 
         # State-values: One for each item in the batch.
         expected_state_value_output = np.matmul(
@@ -323,29 +362,35 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
             policy_params["shared-value-function-policy/value-function-node/dense-layer/dense/kernel"]
         )
         expected_state_value_output_unfolded = np.reshape(expected_state_value_output, newshape=(2, 3, 1))
-        test.test(("get_state_values", states),
-                  expected_outputs=dict(state_values=expected_state_value_output_unfolded), decimals=5)
+        test.test(
+            ("get_state_values", states, ["state_values"]),
+            expected_outputs=dict(state_values=expected_state_value_output_unfolded),
+            decimals=5
+        )
 
         test.test(
-            ("get_state_values_logits_parameters_log_probs", states, ["state_values", "logits"]),
+            ("get_state_values_adapter_outputs_and_parameters", states, ["state_values", "adapter_outputs"]),
             expected_outputs=dict(
-                state_values=expected_state_value_output_unfolded, logits=expected_action_layer_output_unfolded
-            ), decimals=5
+                state_values=expected_state_value_output_unfolded, adapter_outputs=expected_action_layer_output_unfolded
+            ),
+            decimals=5
         )
 
         # Parameter (probabilities). Softmaxed logits.
         expected_parameters_output = tuple([
-            softmax(expected_action_layer_output_unfolded[0], axis=-1),
-            softmax(expected_action_layer_output_unfolded[1], axis=-1),
+            np.maximum(softmax(expected_action_layer_output_unfolded[0], axis=-1), SMALL_NUMBER),
+            np.maximum(softmax(expected_action_layer_output_unfolded[1], axis=-1), SMALL_NUMBER),
             dict(
-                a=softmax(expected_action_layer_output_unfolded[2]["a"], axis=-1)
+                a=np.maximum(softmax(expected_action_layer_output_unfolded[2]["a"], axis=-1), SMALL_NUMBER)
             )
         ])
-        test.test(("get_logits_parameters_log_probs", states, ["logits", "parameters"]),
-                  expected_outputs=dict(
-                      logits=expected_action_layer_output_unfolded,
-                      parameters=expected_parameters_output
-                  ), decimals=5)
+        test.test(
+            ("get_adapter_outputs_and_parameters", states, ["adapter_outputs", "parameters"]),
+            expected_outputs=dict(
+                adapter_outputs=expected_action_layer_output_unfolded, parameters=expected_parameters_output
+            ),
+            decimals=5
+        )
 
         print("Probs: {}".format(expected_parameters_output))
 
@@ -358,39 +403,42 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         ])
         test.test(("get_action", states, ["action"]), expected_outputs=dict(action=expected_actions))
 
-        # Action log-probs.
-        expected_action_log_prob_output = tuple([
-            np.log(np.array([[
-                expected_parameters_output[0][0][0][expected_actions[0][0][0]],
-                expected_parameters_output[0][0][1][expected_actions[0][0][1]],
-                expected_parameters_output[0][0][2][expected_actions[0][0][2]],
-            ], [
-                expected_parameters_output[0][1][0][expected_actions[0][1][0]],
-                expected_parameters_output[0][1][1][expected_actions[0][1][1]],
-                expected_parameters_output[0][1][2][expected_actions[0][1][2]],
-            ]])),
-            np.log(np.array([[
-                expected_parameters_output[1][0][0][expected_actions[1][0][0]],
-                expected_parameters_output[1][0][1][expected_actions[1][0][1]],
-                expected_parameters_output[1][0][2][expected_actions[1][0][2]],
-            ], [
-                expected_parameters_output[1][1][0][expected_actions[1][1][0]],
-                expected_parameters_output[1][1][1][expected_actions[1][1][1]],
-                expected_parameters_output[1][1][2][expected_actions[1][1][2]],
-            ]])),
-            dict(a=np.log(np.array([[
-                expected_parameters_output[2]["a"][0][0][expected_actions[2]["a"][0][0]],
-                expected_parameters_output[2]["a"][0][1][expected_actions[2]["a"][0][1]],
-                expected_parameters_output[2]["a"][0][2][expected_actions[2]["a"][0][2]],
-            ], [
-                expected_parameters_output[2]["a"][1][0][expected_actions[2]["a"][1][0]],
-                expected_parameters_output[2]["a"][1][1][expected_actions[2]["a"][1][1]],
-                expected_parameters_output[2]["a"][1][2][expected_actions[2]["a"][1][2]],
-            ]])))
-        ])
-        test.test(("get_action_log_probs", [states, expected_actions]), expected_outputs=dict(
-            action_log_probs=expected_action_log_prob_output, logits=expected_action_layer_output_unfolded
-        ), decimals=5)
+        out = test.test(("get_action_and_log_likelihood", states))
+        action = out["action"]
+        llh = out["log_likelihood"]
+
+        # Action log-likelihood.
+        expected_action_llh_output = np.log(np.array([[
+            expected_parameters_output[0][0][0][action[0][0][0]],
+            expected_parameters_output[0][0][1][action[0][0][1]],
+            expected_parameters_output[0][0][2][action[0][0][2]],
+        ], [
+            expected_parameters_output[0][1][0][action[0][1][0]],
+            expected_parameters_output[0][1][1][action[0][1][1]],
+            expected_parameters_output[0][1][2][action[0][1][2]],
+        ]])) + np.log(np.array([[
+            expected_parameters_output[1][0][0][action[1][0][0]],
+            expected_parameters_output[1][0][1][action[1][0][1]],
+            expected_parameters_output[1][0][2][action[1][0][2]],
+        ], [
+            expected_parameters_output[1][1][0][action[1][1][0]],
+            expected_parameters_output[1][1][1][action[1][1][1]],
+            expected_parameters_output[1][1][2][action[1][1][2]],
+        ]])) + np.log(np.array([[
+            expected_parameters_output[2]["a"][0][0][action[2]["a"][0][0]],
+            expected_parameters_output[2]["a"][0][1][action[2]["a"][0][1]],
+            expected_parameters_output[2]["a"][0][2][action[2]["a"][0][2]],
+        ], [
+            expected_parameters_output[2]["a"][1][0][action[2]["a"][1][0]],
+            expected_parameters_output[2]["a"][1][1][action[2]["a"][1][1]],
+            expected_parameters_output[2]["a"][1][2][action[2]["a"][1][2]],
+        ]]))
+        test.test(
+            ("get_log_likelihood", [states, action]), expected_outputs=dict(
+                log_likelihood=expected_action_llh_output, adapter_outputs=expected_action_layer_output_unfolded
+            ), decimals=5
+        )
+        recursive_assert_almost_equal(expected_action_llh_output, llh, decimals=5)
 
         # Deterministic sample.
         out = test.test(("get_deterministic_action", states), expected_outputs=None)
@@ -428,10 +476,10 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
             a=Tuple(IntBox(2), IntBox(3)),
             b=Dict(dict(ba=IntBox(4)))
         ), add_batch_rank=True)
-        flat_float_action_space = Dict(dict(
-            a=Tuple(FloatBox(shape=(2,)), FloatBox(shape=(3,))),
-            b=Dict(dict(ba=FloatBox(shape=(4,))))
-        ), add_batch_rank=True)
+        #flat_float_action_space = Dict(dict(
+        #    a=Tuple(FloatBox(shape=(2,)), FloatBox(shape=(3,))),
+        #    b=Dict(dict(ba=FloatBox(shape=(4,))))
+        #), add_batch_rank=True)
 
         # Policy with dueling logic.
         policy = DuelingPolicy(
@@ -448,7 +496,7 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         test = ComponentTest(
             component=policy,
             input_spaces=dict(
-                nn_input=nn_input_space,
+                nn_inputs=nn_input_space,
                 actions=action_space,
                 #logits=flat_float_action_space,
                 #parameters=flat_float_action_space
@@ -463,7 +511,7 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         expected_nn_output = relu(np.matmul(
             nn_input, policy_params["dueling-policy/test-network/hidden-layer/dense/kernel"]), 0.2
         )
-        test.test(("get_nn_output", nn_input), expected_outputs=dict(output=expected_nn_output), decimals=5)
+        test.test(("get_nn_outputs", nn_input), expected_outputs=expected_nn_output, decimals=5)
 
         # Raw action layer output.
         expected_raw_advantages = dict(
@@ -488,16 +536,17 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
                 ), 0.2), policy_params["dueling-policy/action-adapter-2/action-network/action-layer/dense/kernel"]
             ))
         )
-        test.test(("get_action_layer_output", nn_input), expected_outputs=dict(output=expected_raw_advantages),
-                  decimals=5)
 
         # Single state values.
         expected_state_values = np.matmul(relu(np.matmul(
             expected_nn_output,
             policy_params["dueling-policy/dense-layer-state-value-stream/dense/kernel"]
         )), policy_params["dueling-policy/state-value-node/dense/kernel"])
-        test.test(("get_state_values", nn_input), expected_outputs=dict(state_values=expected_state_values),
-                  decimals=5)
+        test.test(
+            ("get_state_values", nn_input, ["state_values"]),
+            expected_outputs=dict(state_values=expected_state_values),
+            decimals=5
+        )
 
         # State-values: One for each item in the batch.
         expected_q_values_output = dict(
@@ -510,23 +559,42 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
             b=dict(ba=expected_state_values + expected_raw_advantages["b"]["ba"] - np.mean(
                 expected_raw_advantages["b"]["ba"], axis=-1, keepdims=True))
         )
-        test.test(("get_logits_parameters_log_probs", nn_input, ["logits"]), expected_outputs=dict(
-            logits=expected_q_values_output
-        ), decimals=5)
+        test.test(
+            ("get_adapter_outputs", nn_input),
+            expected_outputs=dict(
+                adapter_outputs=expected_q_values_output, nn_outputs=expected_nn_output,
+                advantages=expected_raw_advantages, q_values=expected_q_values_output
+            ),
+            decimals=5
+        )
+
+        test.test(
+            ("get_adapter_outputs_and_parameters", nn_input, ["adapter_outputs"]),
+            expected_outputs=dict(adapter_outputs=expected_q_values_output),
+            decimals=5
+        )
 
         # Parameter (probabilities). Softmaxed q_values.
         expected_parameters_output = dict(
-            a=(softmax(expected_q_values_output["a"][0], axis=-1), softmax(expected_q_values_output["a"][1], axis=-1)),
-            b=dict(ba=softmax(expected_q_values_output["b"]["ba"], axis=-1))
+            a=(
+                np.maximum(softmax(expected_q_values_output["a"][0], axis=-1), SMALL_NUMBER),
+                np.maximum(softmax(expected_q_values_output["a"][1], axis=-1), SMALL_NUMBER)
+            ),
+            b=dict(ba=np.maximum(softmax(expected_q_values_output["b"]["ba"], axis=-1), SMALL_NUMBER))
         )
         expected_log_probs_output = dict(
             a=(np.log(expected_parameters_output["a"][0]),
                np.log(expected_parameters_output["a"][1])),
             b=dict(ba=np.log(expected_parameters_output["b"]["ba"]))
         )
-        test.test(("get_logits_parameters_log_probs", nn_input, ["logits", "parameters", "log_probs"]),
-                  expected_outputs=dict(logits=expected_q_values_output, parameters=expected_parameters_output,
-                                        log_probs=expected_log_probs_output), decimals=5)
+        test.test(
+            ("get_adapter_outputs_and_parameters", nn_input, ["adapter_outputs", "parameters", "log_probs"]),
+            expected_outputs=dict(
+                adapter_outputs=expected_q_values_output, parameters=expected_parameters_output,
+                log_probs=expected_log_probs_output
+            ),
+            decimals=5
+        )
 
         print("Probs: {}".format(expected_parameters_output))
 
@@ -537,26 +605,30 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         )
         test.test(("get_action", nn_input, ["action"]), expected_outputs=dict(action=expected_actions))
 
-        # Action log-probs.
-        expected_action_log_prob_output = dict(
-            a=(np.array([
-                expected_log_probs_output["a"][0][0][expected_actions["a"][0][0]],
-                expected_log_probs_output["a"][0][1][expected_actions["a"][0][1]],
-                expected_log_probs_output["a"][0][2][expected_actions["a"][0][2]],
-            ]),
-               np.array([
-                   expected_log_probs_output["a"][1][0][expected_actions["a"][1][0]],
-                   expected_log_probs_output["a"][1][1][expected_actions["a"][1][1]],
-                   expected_log_probs_output["a"][1][2][expected_actions["a"][1][2]],
-               ])),
-            b=dict(ba=np.array([
-                expected_log_probs_output["b"]["ba"][0][expected_actions["b"]["ba"][0]],
-                expected_log_probs_output["b"]["ba"][1][expected_actions["b"]["ba"][1]],
-                expected_log_probs_output["b"]["ba"][2][expected_actions["b"]["ba"][2]],
-            ]))
+        out = test.test(("get_action_and_log_likelihood", nn_input))
+        action = out["action"]
+        llh = out["log_likelihood"]
+
+        # Action log-likelihood.
+        expected_action_llh_output = np.array([
+            expected_log_probs_output["a"][0][0][action["a"][0][0]],
+            expected_log_probs_output["a"][0][1][action["a"][0][1]],
+            expected_log_probs_output["a"][0][2][action["a"][0][2]],
+        ]) + np.array([
+            expected_log_probs_output["a"][1][0][action["a"][1][0]],
+            expected_log_probs_output["a"][1][1][action["a"][1][1]],
+            expected_log_probs_output["a"][1][2][action["a"][1][2]],
+        ]) + np.array([
+            expected_log_probs_output["b"]["ba"][0][action["b"]["ba"][0]],
+            expected_log_probs_output["b"]["ba"][1][action["b"]["ba"][1]],
+            expected_log_probs_output["b"]["ba"][2][action["b"]["ba"][2]],
+        ])
+        test.test(
+            ("get_log_likelihood", [nn_input, action]), expected_outputs=dict(
+                log_likelihood=expected_action_llh_output, adapter_outputs=expected_q_values_output
+            ), decimals=5
         )
-        test.test(("get_action_log_probs", [nn_input, expected_actions]),
-                  expected_outputs=dict(action_log_probs=expected_action_log_prob_output, logits=expected_q_values_output), decimals=5)
+        recursive_assert_almost_equal(expected_action_llh_output, llh, decimals=5)
 
         # Stochastic sample.
         out = test.test(("get_stochastic_action", nn_input), expected_outputs=None)
@@ -584,4 +656,3 @@ class TestPoliciesOnContainerActions(unittest.TestCase):
         self.assertTrue(out["entropy"]["a"][1].shape == (3,))
         self.assertTrue(out["entropy"]["b"]["ba"].dtype == np.float32)
         self.assertTrue(out["entropy"]["b"]["ba"].shape == (3,))
-
