@@ -1,4 +1,4 @@
-# Copyright 2018 The RLgraph Authors, All Rights Reserved.
+# Copyright 2018/2019 The RLgraph Authors, All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,12 +18,13 @@ from __future__ import division
 from __future__ import print_function
 
 import inspect
+
 import numpy as np
 
 from rlgraph.spaces.space_utils import get_space_from_op
 from rlgraph.utils import convert_dtype
-from rlgraph.utils.rlgraph_errors import RLGraphError, RLGraphAPICallParamError
 from rlgraph.utils.ops import FlattenedDataOp, flatten_op, unflatten_op, is_constant
+from rlgraph.utils.rlgraph_errors import RLGraphError, RLGraphAPICallParamError
 
 
 class DataOpRecord(object):
@@ -56,7 +57,7 @@ class DataOpRecord(object):
         self.op = op
         # Some instruction on how to derive the `op` property of this record (other than just: pass along).
         # e.g. "key-lookup: [some key]" if previous op is a DataOpDict.
-        self.op_instructions = None
+        self.op_instructions = dict()
         # Whether the op in this record is one of the last in the graph (a core API-method returned op).
         self.is_terminal_op = False
 
@@ -86,18 +87,21 @@ class DataOpRecord(object):
 
     def __getitem__(self, key):
         """
-        Returns new DataOpRecordColumn with
+        Creates new DataOpRecordColumn with a single op-rec pointing via its `op_instruction` dict
+        back to the previous column's op-rec (this one). This can be used to instruct the building process to
+        do tuple/dict lookups during the build process to implicitly use ContainerMerger/Splitter helper components
+        for a more intuitive handling of DataOpRecords within Component API methods.
 
         Args:
             key (str): The lookup key
 
         Returns:
-
+            A new DataOpRecord with the op_instructions set to do a tuple (idx) or dict (key) lookup at build time.
         """
         column = DataOpRecordColumn(
             self.column.component, args=[self]
         )
-        column.op_records[0].op_instructions = "key-lookup:{}".format(key)
+        column.op_records[0].op_instructions["key-lookup"] = key
         return column.op_records[0]
 
     def __str__(self):
@@ -123,10 +127,29 @@ class DataOpRecordColumn(object):
         if num_op_records is None:
             self.op_records = []
             if args is not None:
+                args = list(args)
                 for i in range(len(args)):
                     if args[i] is None:
                         continue
                     op_rec = DataOpRecord(op=None, column=self, position=i)
+
+                    # Dict instead of a DataOpRecord -> Translate on the fly into a DataOpRec held by a
+                    # ContainerMerger Component.
+                    if isinstance(args[i], dict):
+                        first_element = next(iter(args[i].values()))
+                        if isinstance(first_element, DataOpRecord):
+                            merger_component = first_element.column.component.get_helper_component(
+                                "container-merger", _args=list(args[i].keys())
+                            )
+                            args[i] = merger_component.merge(*list(args[i].values()))
+                    # Tuple instead of a DataOpRecord -> Translate on the fly into a DataOpRec held by a
+                    # ContainerMerger Component.
+                    elif isinstance(args[i], tuple) and isinstance(args[i][0], DataOpRecord):
+                        merger_component = args[i][0].column.component.get_helper_component(
+                            "container-merger", _args=len(args[i])
+                        )
+                        args[i] = merger_component.merge(*args[i])
+
                     # If incoming is an op-rec -> Link them.
                     if isinstance(args[i], DataOpRecord):
                         op_rec.previous = args[i]
@@ -185,6 +208,8 @@ class DataOpRecordColumn(object):
         kwargs = {}
         for op_rec in self.op_records:
             if op_rec.kwarg is None:
+                if op_rec.position > len(args):
+                    args += [None] * (op_rec.position - len(args))
                 args.append(op_rec)
             else:
                 kwargs[op_rec.kwarg] = op_rec
@@ -218,7 +243,9 @@ class DataOpRecordColumnIntoGraphFn(DataOpRecordColumn):
     to.
     """
     def __init__(self, component, graph_fn, flatten_ops=False,
-                 split_ops=False, add_auto_key_as_first_param=False, args=None, kwargs=None):
+                 split_ops=False, add_auto_key_as_first_param=False,
+                 requires_variable_completeness=False,
+                 args=None, kwargs=None):
         # TODO: need to pass in input-arg name information so we can switch to kwargs if some default arg not given.
         super(DataOpRecordColumnIntoGraphFn, self).__init__(
             component=component, args=args, kwargs=kwargs
@@ -230,6 +257,7 @@ class DataOpRecordColumnIntoGraphFn(DataOpRecordColumn):
         self.flatten_ops = flatten_ops
         self.split_ops = split_ops
         self.add_auto_key_as_first_param = add_auto_key_as_first_param
+        self.requires_variable_completeness = requires_variable_completeness
 
         # The column after passing this one through the graph_fn.
         self.out_graph_fn_column = None
@@ -295,14 +323,19 @@ class DataOpRecordColumnIntoGraphFn(DataOpRecordColumn):
         assert all(op is not None for op in ops)  # just make sure
 
         # Collect FlattenedDataOp for checking their keys (must match).
-        flattened = [op.items() for op in ops if len(op) > 1 or "" not in op]
+        flattened = []
+        for op in ops:
+            if isinstance(op, dict) and (len(op) > 1 or "" not in op):
+                flattened.append(op)
+
         # If it's more than 1, make sure they match. If they don't match: raise Error.
         if len(flattened) > 1:
             # Loop through the non-first ones and make sure all keys match vs the first one.
+            lead_arg_dict = flattened[0]
             for other in flattened[1:]:
                 other_arg_iter = iter(other)
-                for key, value in flattened[0]:
-                    k_other, v_other = next(other_arg_iter)
+                for key in lead_arg_dict.keys():
+                    k_other = next(other_arg_iter)
                     if key != k_other:  # or get_shape(v_other) != get_shape(value):
                         raise RLGraphError("ERROR: Flattened ops have a key mismatch ({} vs {})!".format(key, k_other))
 
@@ -316,21 +349,27 @@ class DataOpRecordColumnIntoGraphFn(DataOpRecordColumn):
             for key in guide_op.keys():
                 # Prep input params for a single call.
                 params = [key] if self.add_auto_key_as_first_param is True else []
+                kwargs = {}
                 for op in ops:
-                    params.append(op[key] if key in op else op[""])
-                # Add kwarg_ops
+                    # Check first, do not try to check key into tensor (not iterable):
+                    if isinstance(op, dict):
+                        params.append(op[key] if key in op else op[""])
+                    else:
+                        # E.g. tuple args.
+                        params.append(op)
+
+                # Add kwarg_ops.
                 for kwarg_key, kwarg_op in kwarg_ops.items():
-                    params.append(tuple([
-                        kwarg_key,
-                        kwarg_ops[kwarg_key][key] if key in kwarg_ops[kwarg_key] else kwarg_ops[kwarg_key][""]
-                    ]))
+                    kwargs[kwarg_key] = kwarg_ops[kwarg_key][key] \
+                        if key in kwarg_ops[kwarg_key] else kwarg_ops[kwarg_key][""]
                 # Now do the single call.
-                collected_call_params[key] = params
+                collected_call_params[key] = (params, kwargs)
             return collected_call_params
         # We don't have any container ops: No splitting possible. Return args and kwargs as is.
         else:
-            return tuple(([""] if self.add_auto_key_as_first_param is True else []) + [op[""] for op in ops]),\
-                   {key: value[""] for key, value in kwarg_ops.items()}
+            params = [""] if self.add_auto_key_as_first_param is True else []
+            params += [op[""] if isinstance(op, dict) else op for op in ops]
+            return tuple(params), {key: value[""] for key, value in kwarg_ops.items()}
 
     @staticmethod
     def unflatten_output_ops(*ops):
@@ -417,7 +456,8 @@ class APIMethodRecord(object):
     def __init__(self, func, wrapper_func, name,
                  component=None, must_be_complete=True, ok_to_overwrite=False,
                  is_graph_fn_wrapper=False, is_class_method=True,
-                 flatten_ops=False, split_ops=False, add_auto_key_as_first_param=False):
+                 flatten_ops=False, split_ops=False, add_auto_key_as_first_param=False,
+                 requires_variable_completeness=False):
         """
         Args:
             func (callable): The actual API-method (callable).
@@ -440,6 +480,8 @@ class APIMethodRecord(object):
         self.split_ops = split_ops
         self.add_auto_key_as_first_param = add_auto_key_as_first_param
 
+        self.requires_variable_completeness = requires_variable_completeness
+
         # List of the input-parameter names (str) of this API-method.
         self.input_names = []
         # Name of the *args arg (usually "args") w/o the *.
@@ -448,6 +490,9 @@ class APIMethodRecord(object):
         self.kwargs_name = None
         # List of the names of all non-arg/non-kwarg arguments (the ones that come first in the signature).
         self.non_args_kwargs = []
+        # List of the names of all keyword-only arguments (ones that come after *args, but have a default value and
+        # must be set via a keyword).
+        self.keyword_only = []
         # List of args that have a default value.
         self.default_args = []
 
@@ -462,7 +507,10 @@ class APIMethodRecord(object):
         for param in param_list:
             # This param has a default value.
             if param.default != inspect.Parameter.empty:
-                self.non_args_kwargs.append(param.name)
+                if self.args_name is not None:
+                    self.keyword_only.append(param.name)
+                else:
+                    self.non_args_kwargs.append(param.name)
                 self.default_args.append(param.name)
             # *args
             elif param.kind == inspect.Parameter.VAR_POSITIONAL:
@@ -480,7 +528,8 @@ class APIMethodRecord(object):
 
 class GraphFnRecord(object):
     def __init__(self, func, wrapper_func, component=None, is_class_method=True,
-                 flatten_ops=False, split_ops=False, add_auto_key_as_first_param=False):
+                 flatten_ops=False, split_ops=False, add_auto_key_as_first_param=False,
+                 requires_variable_completeness=False):
         self.func = func
         self.wrapper_func = wrapper_func
         self.name = self.func.__name__
@@ -491,6 +540,7 @@ class GraphFnRecord(object):
         self.flatten_ops = flatten_ops
         self.split_ops = split_ops
         self.add_auto_key_as_first_param = add_auto_key_as_first_param
+        self.requires_variable_completeness = requires_variable_completeness
 
         self.in_op_columns = []
         self.out_op_columns = []
@@ -512,7 +562,7 @@ def get_call_param_name(op_rec):
                 param_name = api_method_rec.input_names[op_rec.position]
         # op_rec has name -> Can only be normal arg of one of **kwargs (if any).
         else:
-            if op_rec.kwarg in api_method_rec.non_args_kwargs:
+            if op_rec.kwarg in api_method_rec.non_args_kwargs + api_method_rec.keyword_only:
                 param_name = op_rec.kwarg
             else:
                 if api_method_rec.kwargs_name is None:
