@@ -84,6 +84,23 @@ class DataOpDict(ContainerDataOp, dict):
     we can store them in sets and use them as lookup keys in other dicts.
     Dict() Spaces produce DataOpDicts when methods like `get_variable` are called on them.
     """
+    def map(self, mapping):
+        """
+        Maps this DataOpDict via a given mapping function to another, corresponding DataOpDict where all individual
+        SingleDataOps are mapped to new SingleDataOps.
+
+        Args:
+            mapping (callable): The mapping function to use on each SingeDataOp.
+
+        Returns:
+            DataOpDict: A copy of this DataOpDict, but all SingeDataOps are mapped via the given mapping function.
+        """
+        flattened_self = flatten_op(self)
+        ret = {}
+        for key, value in flattened_self.items():
+            ret[key] = mapping(key, value)
+        return DataOpDict(dict(unflatten_op(ret)))
+
     def __hash__(self):
         """
         Hash based on sequence of sorted items (keys are all strings, values are always other DataOps).
@@ -95,6 +112,23 @@ class DataOpTuple(ContainerDataOp, tuple):
     """
     A simple wrapper for a (possibly nested) tuple that contains other DataOps.
     """
+    def map(self, mapping):
+        """
+        Maps this DataOpTuple via a given mapping function to another, corresponding DataOpTuple where all individual
+        SingleDataOps are mapped to new SingleDataOps.
+
+        Args:
+            mapping (callable): The mapping function to use on each SingeDataOp.
+
+        Returns:
+            DataOpTuple: A copy of this DataOpTuple, but all SingeDataOps are mapped via the given mapping function.
+        """
+        flattened_self = flatten_op(self)
+        ret = {}
+        for key, value in flattened_self.items():
+            ret[key] = mapping(key, value)
+        return DataOpTuple(*unflatten_op(ret))
+
     def __new__(cls, *components):
         if isinstance(components[0], (list, tuple)):
             assert len(components) == 1
@@ -112,7 +146,7 @@ class FlattenedDataOp(DataOp, OrderedDict):
     pass
 
 
-def flatten_op(op, key_scope="", op_tuple_list=None, scope_separator_at_start=True):
+def flatten_op(op, key_scope="", op_tuple_list=None, scope_separator_at_start=True, mapping=None):
     """
     Flattens a single ContainerDataOp or a native python dict/tuple into a FlattenedDataOp with auto-key generation.
 
@@ -130,12 +164,15 @@ def flatten_op(op, key_scope="", op_tuple_list=None, scope_separator_at_start=Tr
 
     # Are we in the non-recursive (first) call?
     if op_tuple_list is None:
-        # Flatten a SingleDataOp -> return FlattenedDataOp with only-key=""
+        # Flatten a SingleDataOp -> return FlattenedDataOp with only-key="".
         if not isinstance(op, (ContainerDataOp, dict, tuple)):
             return FlattenedDataOp([("", op)])
         op_tuple_list = []
         ret = True
 
+    if mapping is not None:
+        op = mapping(op)
+
     if isinstance(op, dict):
         if scope_separator_at_start:
             key_scope += FLATTEN_SCOPE_PREFIX
@@ -144,7 +181,9 @@ def flatten_op(op, key_scope="", op_tuple_list=None, scope_separator_at_start=Tr
         for key in sorted(op.keys()):
             # Make sure we have no double slashes from flattening an already FlattenedDataOp.
             scope = (key_scope[:-1] if len(key) == 0 or key[0] == "/" else key_scope) + key
-            flatten_op(op[key], key_scope=scope, op_tuple_list=op_tuple_list, scope_separator_at_start=True)
+            flatten_op(
+                op[key], key_scope=scope, op_tuple_list=op_tuple_list, scope_separator_at_start=True, mapping=mapping
+            )
     elif isinstance(op, tuple):
         if scope_separator_at_start:
             key_scope += FLATTEN_SCOPE_PREFIX + FLAT_TUPLE_OPEN
@@ -152,9 +191,8 @@ def flatten_op(op, key_scope="", op_tuple_list=None, scope_separator_at_start=Tr
             key_scope += "" + FLAT_TUPLE_OPEN
         for i, c in enumerate(op):
             flatten_op(c, key_scope=key_scope + str(i) + FLAT_TUPLE_CLOSE, op_tuple_list=op_tuple_list,
-                       scope_separator_at_start=True)
+                       scope_separator_at_start=True, mapping=mapping)
     else:
-        assert not isinstance(op, (dict, tuple))
         op_tuple_list.append((key_scope, op))
 
     # Non recursive (first) call -> Return the final FlattenedDataOp.
@@ -243,13 +281,13 @@ def flat_key_lookup(container, flat_key, default=None):
     result = container
     for key in key_sequence:
         mo = re.match(r'^{}(\d+){}$'.format(FLAT_TUPLE_OPEN, FLAT_TUPLE_CLOSE), key)
-        # Tuple
+        # Tuple.
         if mo is not None:
             slot = int(mo.group(1))
             if len(result) > slot and default is not None:
                 return default
             result = result[slot]
-        # Dict
+        # Dict.
         else:
             if key not in result and default is not None:
                 return default
