@@ -19,6 +19,8 @@ from __future__ import print_function
 
 from rlgraph import get_backend
 import numpy as np
+import copy
+
 
 if get_backend() == "pytorch":
     import torch
@@ -42,27 +44,27 @@ class PyTorchVariable(object):
     def get_value(self):
         if get_backend() == "pytorch":
             if isinstance(self.ref, torch.nn.Module):
-                return self.ref.weight
+                return self.ref.weight.detach()
 
     def set_value(self, value):
         if get_backend() == "pytorch":
             if isinstance(self.ref, torch.nn.Module):
-                if isinstance(value, torch.Tensor):
-                    self.ref.weight = torch.nn.Parameter(value, requires_grad=True)
-                elif isinstance(value, torch.nn.Parameter):
-                    self.ref.weight = value
+                if isinstance(value, torch.nn.Parameter):
+                    self.ref.weight = copy.deepcopy(value)
+                elif isinstance(value, torch.Tensor):
+                    self.ref.weight = torch.nn.Parameter(copy.deepcopy(value), requires_grad=True)
                 else:
                     raise ValueError("Value assigned must be torch.Tensor or Parameter but is {}.".format(
                         type(value)
                     ))
 
 
-def pytorch_one_hot(tensor, depth=0):
+def pytorch_one_hot(index_tensor, depth=0):
     """
     One-hot utility function for PyTorch.
 
     Args:
-        tensor (torch.Tensor): The input to be one-hot.
+        index_tensor (torch.Tensor): The input to be one-hot.
         depth (int): The max. number to be one-hot encoded (size of last rank).
 
     Returns:
@@ -70,22 +72,15 @@ def pytorch_one_hot(tensor, depth=0):
     """
     if get_backend() == "pytorch":
         # Do converts.
-        if isinstance(tensor, torch.FloatTensor):
-            tensor = tensor.long()
-        if isinstance(tensor, torch.IntTensor):
-            tensor = tensor.long()
-        # Un-squeeze 1d.
-        if tensor.dim() == 1:
-            tensor = tensor.unsqueeze(-1)
-        # print("one hot input: {}, dim: {}, shape {}, type {}".format(
-        #     tensor, tensor.dim(), tensor.shape, tensor.dtype
-        # ))
-        # print("tensor.shape[0]", tensor.shape[0])
-        tensor_one_hot = torch.FloatTensor(tensor.shape[0], depth)
-        tensor_one_hot.zero_()
-        tensor_one_hot.scatter_(1, tensor, 1)
+        if isinstance(index_tensor, torch.FloatTensor):
+            index_tensor = index_tensor.long()
+        if isinstance(index_tensor, torch.IntTensor):
+            index_tensor = index_tensor.long()
 
-        return tensor_one_hot
+        out = torch.zeros(index_tensor.size() + torch.Size([depth]))
+        dim = len(index_tensor.size())
+        index = index_tensor.unsqueeze(-1)
+        return out.scatter_(dim, index, 1)
 
 
 def pytorch_tile(tensor, n_tile, dim=0):
@@ -139,14 +134,17 @@ if get_backend() == "pytorch":
         padding options like TF.
         """
         def __init__(self, in_channels, out_channels, kernel_size, bias=True, stride=1,
-                     padding_layer=torch.nn.ReflectionPad2d):
-            super().__init__()
+                     transpose=False, padding_layer=torch.nn.ReflectionPad2d):
+            super(SamePaddedConv2d, self).__init__()
             ka = kernel_size // 2
             kb = ka - 1 if kernel_size % 2 == 0 else ka
 
-            self.layer = torch.nn.Conv2d(in_channels, out_channels, kernel_size, bias=bias, stride=stride)
+            if transpose is True:
+                self.layer = torch.nn.ConvTranspose2d(in_channels, out_channels, kernel_size, bias=bias, stride=stride)
+            else:
+                self.layer = torch.nn.Conv2d(in_channels, out_channels, kernel_size, bias=bias, stride=stride)
             self.net = torch.nn.Sequential(
-                padding_layer((ka,kb,ka,kb)),
+                padding_layer((ka, kb, ka, kb)),
                 self.layer
             )
 
